/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.plugin.parameter.tsf.zeitdauer;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.TreeMap;
import java.util.Map.Entry;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.kalender.ZeitDauerGranularitaet;

public class MyZeitDauer
{
	/*
	 * Definition der Zeitklassen.
	 */
	private ZeitKlasse _jahr = new ZeitKlasse("Jahr", "Jahre", "J");
	private ZeitKlasse _monat = new ZeitKlasse("Monat", "Monate", "M");
	private ZeitKlasse _tag = new ZeitKlasse("Tag", "Tage", "T");
	private ZeitKlasse _stunde = new ZeitKlasse("Stunde", "Stunden", "h");
	private ZeitKlasse _minute = new ZeitKlasse("Minute", "Minuten", "m");
	private ZeitKlasse _sekunde = new ZeitKlasse("Sekunde", "Sekunden", "s");
	private ZeitKlasse _tausendstel = new ZeitKlasse("Tausendstel",
			"Tausendstel", "t");

	/**
	 * Speichert die verfgbaren {@link ZeitKlasse}n.
	 */
	private ZeitKlasse[] _zeitKlassen = new ZeitKlasse[] { _jahr, _monat, _tag,
			_stunde, _minute, _sekunde, _tausendstel };

	/**
	 * Ausgangszeitpunkt.
	 */
	private long _ausgangsZeitpunkt = new Date().getTime();

	/**
	 * Endzeitpunkg
	 */
	private long _endZeitPunkt = new Date().getTime();

	/**
	 * Map der gltigen Zeichen
	 */
	private TreeMap<String, String> _tmGueltigeZeichen = null;

	/**
	 * Gibt an, ob auch Text editiert werden darf.
	 */
	private boolean _allesEditierbar = true;

	/**
	 * Kurzdarstellung
	 */
	private boolean _kurzdarstellung = false;

	/**
	 * Flag, zeigt an ob Jahre dargestellt werden sollen.
	 */
	protected boolean _jahreDarstellen = true;

	/**
	 * Flag, zeigt an ob Monate dargestellt werden sollen.
	 */
	protected boolean _monateDarstellen = true;

	/**
	 * Flag, zeigt an ob Tage dargestellt werden sollen.
	 */
	protected boolean _tageDarstellen = true;

	/**
	 * Flag, zeigt an ob Stunden dargestellt werden sollen.
	 */
	protected boolean _stundenDarstellen = true;

	/**
	 * Flag, zeigt an ob Minuten dargestellt werden sollen.
	 */
	protected boolean _minutenDarstellen = true;

	/**
	 * Flag, zeigt an ob Sekunden dargestellt werden sollen.
	 */
	protected boolean _sekundenDarstellen = true;

	/**
	 * Flag, zeigt an ob Tausendstel dargestellt werden sollen.
	 */
	protected boolean _tausendstelDarstellen = true;

	/**
	 * Konstruktor.
	 * 
	 * @param ausgangsZeitpunkt
	 *          Ausgangszeitpunkt
	 */
	public MyZeitDauer(long ausgangsZeitpunkt)
	{
		this._ausgangsZeitpunkt = ausgangsZeitpunkt;
		init();
	}

	/**
	 * Konstruktor. Lsst Anpassung der Zeitklassen zu, so dass z.B. auch Min und
	 * Max Werte gesetzt werden knnen.
	 * 
	 * @param ausgangsZeitpunkt
	 *          Ausgangszeitpunkt
	 * @param zeitKlassen
	 *          Angepasste Zeitklassen.
	 */
	public MyZeitDauer(long ausgangsZeitpunkt, ZeitKlasse[] zeitKlassen)
	{
		this._ausgangsZeitpunkt = ausgangsZeitpunkt;
		this._zeitKlassen = zeitKlassen;
		init();
	}

	/**
	 * Prft, ob sich auch die Texte der {@link ZeitKlasse}n editieren lassen
	 * oder nur die Zahlenfelder.
	 * 
	 * @return Liefert <code>true</code>, wenn alles editierbar ist, sonst
	 *         <code>false</code>.
	 */
	public boolean istAllesEditierbar()
	{
		return _allesEditierbar;
	}

	/**
	 * Setzt die Editierbarkeit aller Felder. Es knnen sowohl Zahlen als auch
	 * {@link ZeitKlasse}nfelder editiert werden, oder nur die Zahlenfelder.
	 * 
	 * @param allesEditierbar
	 *          Wenn <code>true</code>, dann kann alles editiert werden, bei
	 *          <code>false</code> knnen nur die Zahlenfelder editiert werden.
	 */
	public void setAllesEditierbar(boolean allesEditierbar)
	{
		this._allesEditierbar = allesEditierbar;
	}

	public boolean istKurzdarstellung()
	{
		return _kurzdarstellung;
	}

	public void setKurzdarstellung(boolean kurzdarstellung)
	{
		this._kurzdarstellung = kurzdarstellung;
	}

	/**
	 * Fhrt die Initialisierung durch
	 */
	private void init()
	{
		// _jahr.setBeschraenkungMax(10);
		// _jahr.setBeschraenkungMin(3);
		erstelleListeGueltigerStrings();
	}

	@SuppressWarnings("unused")
	private void werteAus(String text)
	{

		/*
		 * Leerzeichen: Vor Leerzeichen Zahl => nach Leerzeichen Text? Vor
		 * Leerzeichen Text => nach Leerzeichen Zahl?
		 */

		String[] split = text.split(" ");
		System.out.println("SPLIT: " + split.length);

		for (String string : split)
		{
			System.out.println("=> " + string);
		}

		for (int i = 0; i < split.length; i++)
		{
			if (split[i].matches("\\d*"))
			{
				String zahl = split[i];
				if (i < split.length)
				{
					i++;
					String klasse = split[i];
					ZeitKlasse zugehoerigkeit = bestimmeZeitklassenZugehoerigkeit(klasse);
					if (zugehoerigkeit != null)
					{
						System.out.println("Zahl: " + zahl + " (" + klasse
								+ ") gehrt zu: " + zugehoerigkeit.toString());
					}
				}
			}

		}
	}

	public ZeitKlasse bestimmeZeitklassenZugehoerigkeit(String wert)
	{
		ZeitKlasse re = null;

		for (ZeitKlasse zeitKlasse : _zeitKlassen)
		{
			if (wert.equalsIgnoreCase(zeitKlasse.getBezeichnung()))
			{
				re = zeitKlasse;
				break;
			} else if (wert.equalsIgnoreCase(zeitKlasse.getMehrzahl()))
			{
				re = zeitKlasse;
				break;
			} else if (wert.equals(zeitKlasse.getKurzform()))
			{
				re = zeitKlasse;
				break;
			}
		}

		return re;
	}

	public boolean verifyEingabe(String text)
	{
		boolean re = true;

		for (int i = 0; i < text.length(); i++)
		{
			String zeichen = "" + text.charAt(i);
			// System.out.println("Prfe: " + zeichen + " (" + i + ") => "
			// + verifyEingabe(zeichen, i, text, true));

			if (!verifyEingabe(zeichen, i, text, true))
			{
				re = false;
				break;
			}
		}

		return re;
	}

	/**
	 * 
	 * @param zeichen
	 *          Hinzuzufgendes Zeichen
	 * @param position
	 *          Position, an der das Zeichen hinzugefgt werden soll
	 * @param gesamterText
	 *          Der gesamte Text des jeweiligen Textfeldes
	 * @param allesEditierbar
	 *          Wenn <code>true</code>, kann alles editiert werden, wenn
	 *          <code>false</code> knnen nur Zahlen bearbeitet werden.
	 * @return Liefert <code>true</code> wenn die Eingabe verifiziert wurde,
	 *         ansonsten <code>false</code>.
	 */
	public boolean verifyEingabe(String zeichen, int position,
			String gesamterText, boolean allesEditierbar)
	{
		this._allesEditierbar = allesEditierbar;

		/*
		 * Gltig-Flag
		 */
		boolean gueltig = true;

		// System.out.println("ZEICHEN: \"" + zeichen + "\" Position: " + position);

		if (zeichen.length() > 0)
		{
			/*
			 * Alles (inkl. Texte) editierbar?
			 */
			if (_allesEditierbar)
			{

				// ///////////////////////////
				// ////Allgemeine Regeln//////
				// ///////////////////////////

				/*
				 * Prfen auf gltige Zeichen
				 */
				for (int i = 0; i < zeichen.length(); i++)
				{
					String vlg = "" + zeichen.charAt(i);
					vlg = vlg.toLowerCase();
					if (_tmGueltigeZeichen.containsKey(vlg))
					{
						gueltig = true;
					} else
					{
						gueltig = false;
						break;
					}
				}

				/*
				 * Sicherstellen, dass das erste Zeichen eine Zahl ist.
				 */
				if (position == 0 && gueltig)
				{
					try
					{
						@SuppressWarnings("unused")
						Integer valueOf = Integer.valueOf(zeichen); //Muss bleiben!!!
					} catch (NumberFormatException e)
					{
						// e.printStackTrace();
						gueltig = false;
					}

				}

				/*
				 * Sicherstellen, dass keine zwei Leerzeichen hintereinander eingegeben
				 * werden
				 */
				if (gueltig)
				{
					if (position > 0)
					{
						/*
						 * Auf Vorgngerleerzeichen Prfen
						 */
						String vorgaengerZeichen = "" + gesamterText.charAt(position - 1);

						if ((zeichen.equals(" ")) && (zeichen.equals(vorgaengerZeichen)))
						{
							gueltig = false;
//							System.err.println("Keine zwei Leerzeichen hintereinander");
						}

						/*
						 * Auf Nachfolgerleerzeichen (also 2 Leerzeichen hintereinander)
						 * prfen
						 */
						if (gueltig)
						{
							if (position < gesamterText.length())
							{
								// String pr = "" + gesamterText.charAt(position);
								// if (pr.equals(" "))
								// {
								// gueltig = false;
								// System.err.println("Keine zwei Leerzeichen nacheinander");
								// }
								//							

								String zeichenAktuell = "" + gesamterText.charAt(position);
								if (position + 1 < gesamterText.length())
								{
									String zeichenNachfolgend = ""
											+ gesamterText.charAt(position + 1);
									if ((zeichenAktuell.equals(" "))
											&& (zeichenNachfolgend.equals(" ")))
									{
										gueltig = false;
										System.err.println("Keine zwei Leerzeichen nacheinander");
									}
								}

							}
						}
					}

				}

				/*
				 * Vor einem Buchstaben darf keine Zahl, vor einer Zahl kein Buchstabe
				 * stehen
				 */
				if (gueltig)
				{
					if (gesamterText.length() > 2)
					{
						if ((position - 1) > 0)
						{
							String vglVorgaenger = "" + gesamterText.charAt(position - 1);

							if (zeichen.matches("\\d") && vglVorgaenger.matches("[a-zA-Z]"))
							{
								gueltig = false;
							} else if (zeichen.matches("[a-zA-Z]")
									&& vglVorgaenger.matches("\\d"))
							{
								gueltig = false;
							}
						}
					}
				}

				/*
				 * Nach einer Zeitklasse muss eine Zahl folgen. Nach einer Zahl muss
				 * eine Zeitklasse folgen.
				 */
				if (gueltig)
				{
					if (position > 0) // Stellt sicher, dass die erste Zahl auch
					// nachtrglich gesetzt werden kann.
					{
						if (gesamterText.length() > 2)
						{
							/*
							 * Ist Vorgnger ein Leerzeichen?
							 */
							String vglLeerzeichen = "" + gesamterText.charAt(position - 1);
							if (vglLeerzeichen.equals(" "))
							{
								String vglVorgaenger = "" + gesamterText.charAt(position - 2);

								if (zeichen.matches("\\d") && vglVorgaenger.matches("\\d"))
								{
									gueltig = false;
								} else if (zeichen.matches("[a-zA-Z]")
										&& vglVorgaenger.matches("[a-zA-Z]"))
								{
									gueltig = false;
								}

							}
						}
					}
				}

				// //////////////////////////////
				// ////Spezielle Regeln fr//////
				// ////die Zeichenklassen //////
				// //////////////////////////////

				/*
				 * Prfen, ob die eingegebene Zeichenklasse gltig ist - Verwendete
				 * Zeichen
				 */
				if (gueltig)
				{
					String vgl = null;
					if (zeichen.matches("[a-zA-Z]"))
					{
						// Ist Vorgnger Leerzeichen
						String vorg = "" + gesamterText.charAt(position - 1);
						if (vorg.equals(" "))
						{
							vgl = zeichen;
						} else
						{
							int startOffset = -1;
							for (int i = position - 1; i > 0; i--)
							{
								String wert = "" + gesamterText.charAt(i);
								if (wert.equals(" "))
								{
									startOffset = i + 1;
									break;
								}
							}

							// System.out.println("Start: " + startOffset + " Ende: " +
							// position);
							vgl = gesamterText.substring(startOffset, position) + zeichen;

						}

						// System.out.println("Vgl: " + vgl);

						if (!pruefeVorkommenInZeitKlassen(vgl))
						{
							gueltig = false;
						}

					}
				}

				/*
				 * Prfen auf korrekte Schreibweise einer Zeitklasse (Case-Insensetive)
				 */
				if (gueltig)
				{
					if (zeichen.matches("[a-zA-Z]"))
					{
						String bestimmeString = bestimmeStringZeitKlassenEingabe(
								gesamterText, zeichen, position);
						// System.out.println("=>BEST: \"" + bestimmeString + "\"");

						@SuppressWarnings("unused")
						boolean istGueltigeZeitklasse = istGueltigeZeitklasse(bestimmeString);
						// System.out.println("==>" + istGueltigeZeitklasse);

						// if (!istGueltigeZeitklasse(bestimmeString))
						// {
						// System.out.println("Gltige ZeitKlasse => " +
						// istGueltigeZeitklasse(bestimmeString));
						// }
					}
				}

				/*
				 * Prfen, ob die Zeichenklasse schon mal am Stck verwendet wurde
				 */

			} else
			{
				/*
				 * Nur Zahlen editierbar public boolean verifyEingabe(String zeichen,
				 * int position, String gesamterText, boolean allesEditierbar)
				 */

				/*
				 * Zeichen ist Zahl?
				 */
				if (zeichen.matches("\\d"))
				{
					String vorZeichen = "" + gesamterText.charAt(position);
					String nachZeichen = "" + gesamterText.charAt(position + 1);

					if (((vorZeichen.equals(" ")) && (nachZeichen.matches("\\d")))
							|| ((vorZeichen.matches("\\d")) && (nachZeichen.equals(" "))))
					{

					} else
					{
						gueltig = false;
					}

				} else
				{
					gueltig = false;
				}

			}

			// SONST
		} else
		{

		}

		return gueltig;
	}

	protected boolean pruefeVorkommenInZeitKlassen(String string)
	{
		boolean re = false;

		for (ZeitKlasse zeitKlasse : _zeitKlassen)
		{
			if (zeitKlasse.getBezeichnung().toLowerCase().contains(
					string.toLowerCase()))
			{
				re = true;
				break;
			} else if (zeitKlasse.getMehrzahl().toLowerCase().contains(
					string.toLowerCase()))
			{
				re = true;
				break;
			} else if (zeitKlasse.getKurzform().toLowerCase().contains(
					string.toLowerCase()))
			{
				re = true;
				break;
			}
		}

		return re;
	}

	/**
	 * Bestimmt die Position des nchsten Zahlenfeldes.
	 * 
	 * @param text
	 *          Zu prfendens Textfeld
	 * @param carretPos
	 *          Aktuelle Position des Carrets
	 * @param aktSelStartPos
	 *          Start-Position der Selektion, wie von {@link Text#getSelection()}
	 *          geliefert.
	 * @param aktSelEndPos
	 *          End-Position der Selektion, wie von {@link Text#getSelection()}
	 *          geliefert.
	 * @return Zu selektierende Position
	 */
	public SelektierInfo bestimmeNaechstesZahlenfeld(String text, int carretPos,
			int aktSelStartPos, int aktSelEndPos)
	{
		// System.out.println("CarretPos: " + carretPos + " SelStart: "
		// + aktSelStartPos + " SelEnde: " + aktSelEndPos);

		SelektierInfo re = null;

		if ((text != null) && (text.length() > 0) && (carretPos >= 0))
		{
			int startPos = -1;
			int endPos = -1;

			int pos = 0;

			/*
			 * Ausgehend von der Carretpositon (Zeichenpostion + 1) wird
			 * sichergestellt, dass das nchste Feld selektiert wird.
			 */
			if (carretPos > 0)
			{
				if (carretPos == text.length())
				{
					pos = 0;
				} else
				{
					pos = carretPos;
				}
			}

			/*
			 * Aktuelles Zeichen ist Zahl? Wenn ja, Anfangsposition der Zahl suchen
			 */
			// System.out.println("POS=>" + pos);
			String startZeichen = "" + text.charAt(pos);
			if (startZeichen.matches("\\d"))
			{
				startPos = pos;

				for (int i = pos; i >= 0; i--)
				{
					startZeichen = "" + text.charAt(i);

					if (startZeichen.matches("\\d"))
					{
						startPos = i;
					} else
					{
						break;
					}
				}
			}

			/*
			 * Wenn noch keine Startposition gefunden wurde, dann StartPosition nach
			 * hinten bestimmen.
			 */
			if (startPos == -1)
			{
				for (int i = pos; i < text.length(); i++)
				{

					startZeichen = "" + text.charAt(i);

					if (!startZeichen.matches("\\d"))
					{

					} else
					{
						startPos = i;
						break;
					}
				}
			}

			/*
			 * Bestimmen der Endposition
			 */
			String endZeichen = "";
			if (startPos >= 0)
			{
				for (int i = startPos; i < text.length(); i++)
				{
					endZeichen = "" + text.charAt(i);
					endPos = i;
					if (!endZeichen.matches("\\d"))
					{
						endPos--;
						break;
					}
				}
			}

			/*
			 * Wenn Start- und Endposition noch nicht bestimmt sind dann ist davon
			 * auszugehen, das wir uns am Ende des Feldes befinden. Um den durchlauf
			 * zu garantieren, wird von vorne angefangen, wenn die CarretPosition > 0
			 * war.
			 */
			if (!((startPos >= 0) && (endPos >= 0)))
			{
				if (carretPos > 0)
				{
					SelektierInfo naechstesZahlenfeld = bestimmeNaechstesZahlenfeld(text,
							0, aktSelStartPos, aktSelEndPos);
					startPos = naechstesZahlenfeld.getStartPos();
					endPos = naechstesZahlenfeld.getEndPos();
				}
			}

			if ((startPos >= 0) && (endPos >= 0))
			{
				// System.out.println("StartPos: " + startPos + "(="
				// + text.charAt(startPos) + ") EndPos: " + endPos + "(="
				// + text.charAt(endPos) + ")");
				re = new SelektierInfo(startPos, endPos, text);
			} else
			{
				System.out.println("Start & Endposition konnten nicht bestimmt werden");
			}

		}

		return re;
	}

	/**
	 * Bestimmt die Position des nchsten Zahlenfeldes.
	 * 
	 * @param text
	 *          Zu prfendens Textfeld
	 * @param carretPos
	 *          Aktuelle Position des Carrets
	 * @param aktSelStartPos
	 *          Start-Position der Selektion, wie von {@link Text#getSelection()}
	 *          geliefert.
	 * @param aktSelEndPos
	 *          End-Position der Selektion, wie von {@link Text#getSelection()}
	 *          geliefert.
	 * @return Zu selektierende Position
	 */
	public SelektierInfo bestimmeVorhergehendesZahlenfeld(String text,
			int carretPos, int aktSelStartPos, int aktSelEndPos)
	{
		SelektierInfo re = null;

		if ((text != null) && (text.length() > 0) && (carretPos >= 0))
		{
			int startPos = -1;
			int endPos = -1;

			int pos = 0;

			/*
			 * Ausgehend von der Carretpositon (Zeichenpostion + 1) wird
			 * sichergestellt, dass das nchste Feld selektiert wird.
			 */
			if (carretPos > 0)
			{
				if (carretPos == text.length())
				{
					pos = 0;
				} else
				{
					pos = carretPos;
				}
			}

			/*
			 * Aktuelles Zeichen ist Zahl? Wenn ja, Anfangsposition der Zahl suchen
			 */

			String startZeichen = null;
			if (pos > 0)
			{
				startZeichen = "" + text.charAt(pos - 1);

				if (startZeichen.matches("\\d"))
				{
					startPos = pos - 1;

					for (int i = pos - 1; i >= 0; i--)
					{
						startZeichen = "" + text.charAt(i);

						if (startZeichen.matches("\\d"))
						{
							startPos = i;
						} else
						{
							break;
						}
					}
				}
			}

			/*
			 * Wenn noch keine Startposition gefunden wurde, dann StartPosition nach
			 * vorne bestimmen.
			 */
			boolean zahlGefunden = false;
			if (startPos == -1)
			{
				for (int i = pos; i >= 0; i--)
				{

					startZeichen = "" + text.charAt(i);

					if (!zahlGefunden)
					{
						if (startZeichen.matches("\\d"))
						{
							startPos = i;
							zahlGefunden = true;
						}
					} else if (zahlGefunden)
					{
						if (startZeichen.matches("\\d"))
						{
							startPos = i;
						} else
						{
							startPos = i + 1;
							break;
						}
					}

				}
			}

			/*
			 * Bestimmen der Endposition
			 */
			String endZeichen = "";
			if (startPos >= 0)
			{
				for (int i = startPos; i < text.length(); i++)
				{
					endZeichen = "" + text.charAt(i);
					endPos = i;
					if (!endZeichen.matches("\\d"))
					{
						endPos--;
						break;
					}
				}
			}

			if ((startPos >= 0) && (endPos >= 0))
			{
				// System.out.println("StartPos: " + startPos + "(="
				// + text.charAt(startPos) + ") EndPos: " + endPos + "(="
				// + text.charAt(endPos) + ")");
				re = new SelektierInfo(startPos, endPos, text);
			} else
			{
				System.out.println("Start & Endposition konnten nicht bestimmt werden");
			}

			// System.out.println("Startpos: " + startPos + " Endpos: " + endPos);
			// System.out.println("SelStart: " + aktSelStartPos + " SelEnde:
			// "+aktSelEndPos);

			/*
			 * Befinden wir uns ganz am Anfang, dann wollen wir wieder von hinten
			 * beginnen.
			 */
			if (aktSelStartPos == startPos && (aktSelEndPos - 1) == endPos)
			{
				int neuPos = 0;
				if (startPos > 0)
				{
					neuPos = startPos - 1;
				} else
				{
					neuPos = text.length() - 1;
				}

				re = bestimmeVorhergehendesZahlenfeld(text, neuPos, neuPos, neuPos);
			}

		}

		return re;
	}

	/**
	 * Bestimmt die Markierung eine Zahl in der direkten nhe des Carrets.
	 * 
	 * @param text
	 *          Text, in dem die Markierung bestimmt werden soll
	 * @param pos
	 *          Position des Carrets
	 * @return Array mit Start und Endposition der Markierung oder
	 *         <code>null</code>, wenn keine Bestimmung statt finden konnte.
	 */
	public int[] bestimmeMarkierung(String text, int pos)
	{
		int[] re = null;

		int startPos = pos;
		int endPos = -1;

		/*
		 * Ist Zeichen Zahl? Wenn ja, dann gehe in der Folge der Zahlen weiter vor.
		 */
		if (startPos > 0)
		{
			for (int i = (pos - 1); i >= 0; i--)
			{
				String zeichen = "" + text.charAt(i);
				// System.out.println("Prfe "+i+": " + zeichen);

				startPos = i;

				if (!zeichen.matches("\\d"))
				{
					i = i + 1;
					startPos = i;
					break;
				}
			}
		}

		// System.out.println("Start: " + startPos);

		if (startPos == pos)
		{
			if (pos < text.length())
			{
				String zeichen = "" + text.charAt(startPos);
				if (!zeichen.matches("\\d"))
				{
					startPos = -1;
				}
			} else
			{
				startPos = -1;
			}
		}

		/*
		 * Bestimmung nach hinten
		 */
		for (int i = pos; i < text.length(); i++)
		{
			String zeichen = "" + text.charAt(i);

			if (!zeichen.matches("\\d"))
			{
				i = i - 1;
				endPos = i;
				break;
			}
		}

		if (endPos == pos)
		{
			String zeichen = "" + text.charAt(endPos);
			if (!zeichen.matches("\\d"))
			{
				endPos = -1;
			}
		}

		/*
		 * Prfen, ob Strings an Start & Endpos auch Zeichen sind
		 */
		if (startPos >= 0)
		{
			String zeichenStartPos = "" + text.charAt(startPos);
			if (!zeichenStartPos.matches("\\d"))
			{
				startPos = -1;
			}
		}

		if ((endPos >= 0) && (endPos < text.length()))
		{
			String zeichenEndPos = "" + text.charAt(endPos);
			if (!zeichenEndPos.matches("\\d"))
			{
				endPos = -1;
			}
		}

		// System.out.println("Pos: " + pos + " StartPos: " + startPos + " EndPos: "
		// + endPos);

		/*
		 * Rckgabewert zusammenstellen:
		 */
		if ((startPos >= 0) && (endPos >= 0))
		{
			re = new int[] { startPos, endPos };
		}

		return re;
	}

	/**
	 * ndert den Wert einer Zahl in dem bergebenen Textfeld.
	 * 
	 * @param text
	 *          Zu bearbeitender Text
	 * @param selectionText
	 *          Selektierter Text im Textfeld
	 * @param carretPos
	 *          Carret-Position, wird am Ende der Selektion im Textfeld postiert
	 * @param wert
	 *          Wert, der zu dem Zahlenfeld addiert / subtrahiert werden soll
	 * @return
	 */
	public SelektierInfo wertAendern(String text, String selectionText,
			int carretPos, int wert)
	{
		SelektierInfo re = null;

		if ((text != null) && (text.length() > 0))
		{
			if ((selectionText != null) && (selectionText.length() > 0))
			{
				// System.out.println("Text: " + text + " Selektiert: " + selectionText
				// + " CarretPos: " + carretPos + " Wert: " + wert);

				int startPos = carretPos - selectionText.length();

				int endPosAlt = startPos + selectionText.length() - 1;
				// System.out
				// .println("StartPos: " + startPos + " EndPosAlt: " + endPosAlt);

				/*
				 * Ist selektierter Text eine Zahl
				 */
				if (selectionText.matches("\\d*"))
				{
					int zahl = Integer.valueOf(selectionText);
					int zahlNeu = zahl + wert;

					/*
					 * Darf nicht kleiner 0 sein
					 */
					if (zahlNeu < 0)
					{
						zahlNeu = 0;
					}

					ZeitKlasse bestimmeZeitKlasse = bestimmeZeitKlasse(text, endPosAlt);
					boolean ver = bestimmeZeitKlasse.verifiziereWert(zahlNeu);

					if (!ver)
					{
						zahlNeu = zahl;
					}

					String strZahlNeu = "" + zahlNeu;

					String ersterTeil = "";
					if (startPos > 0)
					{
						ersterTeil = text.substring(0, startPos);

					}

					String letzterTeil = "";
					if (endPosAlt + 1 < text.length())
					{
						letzterTeil = text.substring(endPosAlt + 1);
					}

					// System.out.println("Erster Teil: \"" + ersterTeil + "\"");
					// System.out.println("Letzer Teil: \"" + letzterTeil + "\"");

					String neuerText = ersterTeil + strZahlNeu + letzterTeil;
					// System.out.println("Kombiniert: \"" + neuerText + "\"");

					int endPosNeu = startPos + strZahlNeu.length() - 1;
					// System.out.println("Endposneu: " + endPosNeu);

					re = new SelektierInfo(startPos, endPosNeu, neuerText);
				}
			}
		}

		return re;
	}

	/**
	 * Bestimmt die {@link ZeitKlasse} eines Textes nach der Endposition einer
	 * Zahl
	 * 
	 * @param text
	 *          Text, an Hand dem die {@link ZeitKlasse} bestimmt werden soll.
	 * @param endPosZahl
	 *          Endposition der letzten Zahl des Wertes der {@link ZeitKlasse}.
	 * @return Liefert die {@link ZeitKlasse}, die an Hand des eingegebenen
	 *         Textes bestimmt werden konnte.
	 */
	protected ZeitKlasse bestimmeZeitKlasse(String text, int endPosZahl)
	{
		ZeitKlasse re = null;

		if (endPosZahl < text.length())
		{
			if (endPosZahl + 1 < text.length())
			{

				int startPos = endPosZahl + 2;
				int endPos = startPos;

				/*
				 * Bereich des Zeitklassen-Strings bestimmen
				 */
				for (int i = startPos; i < text.length(); i++)
				{

					String aktZeichen = "" + text.charAt(i);
					if (aktZeichen.matches("[a-zA-Z]"))
					{
						endPos = i;
					} else
					{
						break;
					}
				}

				String pruefling = text.substring(startPos, endPos + 1);

				/*
				 * Passende Zeitklasse fr diesen String finden
				 */

				for (ZeitKlasse zeitKlasse : _zeitKlassen)
				{
					boolean gefunden = false;

					if (zeitKlasse.getBezeichnung().equalsIgnoreCase(pruefling))
					{
						gefunden = true;
					} else if (zeitKlasse.getMehrzahl().equalsIgnoreCase(pruefling))
					{
						gefunden = true;
					} else if (zeitKlasse.getKurzform().equals(pruefling))
					{
						gefunden = true;
					}

					if (gefunden)
					{
						re = zeitKlasse;
						break;
					}
				}

			}
		}

		return re;
	}

	/**
	 * Liefert eine Liste von Gltigen Zeichen, welche erfasst werden drfen.
	 * 
	 * @return Liste gltiger Zeichen.
	 */
	public TreeMap<String, String> getListeGueltigerStrings()
	{
		return _tmGueltigeZeichen;
	}

	/**
	 * Bestimmt die Zeichen, die eingegeben werden drfen.
	 * 
	 * @return {@link TreeMap} von eingebbaren Zeichen.
	 */
	private TreeMap<String, String> erstelleListeGueltigerStrings()
	{
		if (_tmGueltigeZeichen == null)
		{

			/*
			 * Gltige Zeichen aus den vorhandenen ZeitKlassen erstellen
			 */

			_tmGueltigeZeichen = new TreeMap<String, String>();
			LinkedList<String[]> strgs = new LinkedList<String[]>();
			for (ZeitKlasse zeitKlasse : _zeitKlassen)
			{
				strgs.add(separiereZeichenAusString(zeitKlasse.getBezeichnung()));
				strgs.add(separiereZeichenAusString(zeitKlasse.getMehrzahl()));
				strgs.add(separiereZeichenAusString(zeitKlasse.getKurzform()));
			}

			Iterator<String[]> it = strgs.iterator();
			while (it.hasNext())
			{
				String[] werte = it.next();
				for (String zeichen : werte)
				{
					if (!_tmGueltigeZeichen.containsKey(zeichen))
					{
						_tmGueltigeZeichen.put(zeichen, zeichen);
					}
				}
			}

			/*
			 * Zahlen 0-9 hinzufgen
			 */
			for (int i = 0; i < 10; i++)
			{
				String wert = "" + i;
				if (!_tmGueltigeZeichen.containsKey(wert))
				{
					_tmGueltigeZeichen.put(wert, wert);
				}
			}

			/*
			 * Leerzeichen
			 */
			_tmGueltigeZeichen.put(" ", " ");
		}

		return _tmGueltigeZeichen;
	}

	/**
	 * Liefert ein Array mit den einzelnen Strings eines Strings. Jedes Zeichen
	 * kommt dabei nur ein mal vor.
	 * 
	 * @param wert
	 *          String, dessen Zeichen bestimmt werden sollen.
	 * @return Zeichen des Strings in einem Array
	 */
	private String[] separiereZeichenAusString(String wert)
	{
		String[] re = null;
		HashMap<String, String> hm = new HashMap<String, String>();

		/*
		 * String Parsen
		 */
		for (int i = 0; i < wert.length(); i++)
		{
			String zeichen = "" + wert.charAt(i);
			zeichen = zeichen.toLowerCase();

			if (!hm.containsKey(zeichen))
			{
				hm.put(zeichen, zeichen);
			}
		}

		/*
		 * Rckgabefeld erstellen
		 */
		re = new String[hm.size()];
		Iterator<Entry<String, String>> it = hm.entrySet().iterator();
		int i = 0;
		while (it.hasNext())
		{
			Entry<String, String> entry = it.next();
			re[i] = entry.getValue();
			i++;
		}

		return re;
	}

	/**
	 * Bestimmt an Hand der angegebenen einfuegePosition den bist dato dort
	 * vorhandenen kompletten String, der sich gegen [a-zA-Z] matchen lsst. Er
	 * wird von Anfang bist Ende bestimmt. <br>
	 * Es ist hiermit eine Prfung daraufhin mglich, zu welcher Zeitklasse der
	 * String wohl gehren wird.
	 * 
	 * @param gesammt
	 *          Der gesamte String des Feldes
	 * @param einfuegen
	 *          Einzufgender String
	 * @param einfuegePosition
	 *          Die Position, an der der neue String eingefgt werden soll.
	 * @return Liefert den jeweiligen String oder <code>null</code>, wenn an
	 *         dieser Position keiner gematched werden konnte
	 */
	protected String bestimmeStringZeitKlassenEingabe(String gesammt,
			String einfuegen, int einfuegePosition)
	{
		String re = null;

		int startPosition = -1;
		int endPosition = -1;

		String vorEinfuegePosition = null; // String vor der Einfgepos.
		String nachEinfuegePosition = null; // String nach der Einfgepos.

		if (!(einfuegePosition >= gesammt.length() + 1))
		{
			/*
			 * Nach vorne parsen
			 */
			if (einfuegePosition > 0)
			{
				for (int i = einfuegePosition - 1; i >= 0; i--)
				{
					String pruefling = "" + gesammt.charAt(i);

					if (!pruefling.matches("[a-zA-Z]"))
					{
						startPosition = i + 1;
						break;
					} else
					{
						startPosition = i;
					}
				}
			} else
			{
				startPosition = 0;
			}

			if (startPosition >= 0)
			{
				if (einfuegePosition >= 0)
				{
					vorEinfuegePosition = gesammt.substring(startPosition,
							einfuegePosition);
				}
			}

			/*
			 * Nach hinten Parsen von der einfuegePosition an
			 */
			if (einfuegePosition >= 0)
			{
				if (einfuegePosition < gesammt.length())
				{
					for (int i = einfuegePosition; i < gesammt.length(); i++)
					{
						String pruefling = "" + gesammt.charAt(i);

						if (!pruefling.matches("[a-zA-Z]"))
						{
							endPosition = i - 1;

							break;
						} else
						{
							endPosition = i;
						}
					}

					nachEinfuegePosition = gesammt.substring(einfuegePosition,
							endPosition + 1);
				} else
				{
					nachEinfuegePosition = "";
				}
			}

			if (!(startPosition >= gesammt.length()))
			{

				if (einfuegePosition >= gesammt.length())
				{
					endPosition = einfuegePosition;
				}

				// System.out.println("Start: " + startPosition + " Ende: " +
				// endPosition);

				if (einfuegePosition >= 0)
				{
					vorEinfuegePosition = gesammt.substring(startPosition,
							einfuegePosition);

				}

				// System.out.println("VOR: " + vorEinfuegePosition);
				// System.out.println("NACH: " + nachEinfuegePosition);

				re = vorEinfuegePosition + einfuegen + nachEinfuegePosition;
				// System.out.println("=>RE: " + re);

				if (startPosition >= endPosition)
				{
					re = null;
				}
			}
		}

		/*
		 * Korrektur berflssiges Leerzeichen!!!
		 */
		if (re != null)
		{
			String z = "" + re.charAt(re.length() - 1);

			if (z.equals(" "))
			{
				System.out.println("Z: " + z);
				re = re.substring(0, re.length() - 1);

			}
		}

		return re;
	}

	/**
	 * Prft, ob ein String zu einer {@link ZeitKlasse} zugehrig ist. Bei
	 * Bezeichnung und Mehrzahl der {@link ZeitKlasse} wird dabei Case-Insensetive
	 * geprft.
	 * 
	 * @param wert
	 *          Zu prfender Wert.
	 * @return Liefert <code>true</code>, wenn es sich um eine gltige
	 *         {@link ZeitKlasse} handelt, sonst <code>false</code>.
	 */
	protected boolean istGueltigeZeitklasse(String wert)
	{
		boolean re = false;

		for (ZeitKlasse zeitKlasse : _zeitKlassen)
		{

			String bezeichnung = zeitKlasse.getBezeichnung().toLowerCase();
			String mehrzahl = zeitKlasse.getMehrzahl().toLowerCase();
			String kurz = zeitKlasse.getKurzform();

			String chk = "";

			if (wert != null)
			{
				for (int i = 0; i < wert.length(); i++)
				{
					chk = chk + wert.charAt(i);
				}
			}

			if (bezeichnung.contains(chk.toLowerCase()))
			{
				re = true;
				break;
			}

			if (mehrzahl.contains(chk.toLowerCase()))
			{
				re = true;
				break;
			}

			if (wert.length() == 1)
			{
				if (kurz.equals(chk))
				{
					re = true;
					break;
				}
			}

		}

		return re;
	}

	/**
	 * Bestimmt die gltigen Felder und erzeugt einen im Feld anzeigbaren String.
	 * Fehlerhafte Werde werden separat bermittelt.
	 * 
	 * @param text
	 *          Der zu validierende String.
	 * @return Liefert eine Instanz der Klasse {@link ValidierungsInfo}, welche
	 */
	public ValidierungsInfo validiereFelder(String text)
	{
		ValidierungsInfo re = null;

		/**
		 * Liste der Felder einer Zeitdauer
		 */
		LinkedList<FeldInfo> felder = new LinkedList<FeldInfo>();

		/**
		 * Liste von nicht bestimmbaren Feldern
		 */
		LinkedList<String> nichtBestimmbar = new LinkedList<String>();

		String[] split = text.split(" ");
		if (split.length >= 2)
		{

			for (int i = 0; i < split.length; i++)
			{

				if (split[i].matches("\\d*"))
				{
					int zahl = Integer.valueOf(split[i]);

					if ((i + 1) < (split.length))
					{
						ZeitKlasse zeitKlasse = bestimmeZeitklassenZugehoerigkeit(split[i + 1]);
						if (zeitKlasse != null)
						{
							/*
							 * Kurzform verwenden? Wir an Hand des Strings festgemacht.
							 */
							FeldInfo feldInfo = null;

							if (split[i + 1].length() == 1)
							{
								feldInfo = new FeldInfo(zeitKlasse, true, zahl);
							} else
							{
								feldInfo = new FeldInfo(zeitKlasse, false, zahl);
							}

							if (felder.size() > 0)
							{
								/*
								 * Prfen ob Feld vom gleichen Typ schon existiert
								 */
								boolean schonda = false;
								Iterator<FeldInfo> it = felder.iterator();
								while (it.hasNext())
								{
									FeldInfo vglFeld = it.next();
									if (vglFeld.getTyp().getBezeichnung().equals(
											feldInfo.getTyp().getBezeichnung()))
									{
										schonda = true;
//										System.err.println("=>" + schonda + " = "
//												+ feldInfo.getTyp().getBezeichnung());
										break;
									}
								}

								if (!schonda)
								{
									felder.add(feldInfo);
								} else
								{
//									System.out
//											.println("Folgendes Feld ist als Typ schon vorhanden: "
//													+ feldInfo.getTyp().getBezeichnung());
								}

							} else
							{
								felder.add(feldInfo);
							}

						} else
						{
							/*
							 * Zeitklasse war nicht bestimmbar
							 */

							// String meldung = "Typ nicht bestimmbar: " + split[i] + " -> "
							// + split[i + 1];
							String meldung = split[i] + " " + split[i + 1];

//							System.err.println(meldung);
							nichtBestimmbar.add(meldung);
						}

						i = i + 1;
					} else
					{
						/*
						 * Kein Nachfolgerfeld fr eine Zahl
						 */
						nichtBestimmbar.add(split[i]);
					}
				}
			}
		}

		/*
		 * Erzeugung des Rckgabewertes
		 */
		re = new ValidierungsInfo(felder, nichtBestimmbar);

		/*
		 * Gerade Feldanzahl? Wenn nein eine entsprechende sonstige Meldung setzen.
		 */
		if ((split.length % 2) > 0)
		{
			String msg = "Die Anzahl der Felder muss gerade sein und dem Muster Zahl Leerzeichen\n"
					+ "Zeitklasse folgen. Das erste Feld muss eine Zahl sein und nach dem\n"
					+ "letzten Zeitklassenfeld darf keine Zahl folgen.";

			re.setSonstigeInfos(msg);
		}

		return re;

	}

	public class ValidierungsInfo
	{
		private LinkedList<FeldInfo> _feldInfos = null;
		private LinkedList<String> _nichtBestimmbareFelder = null;
		private String _sonstigeInfos = "";

		public ValidierungsInfo(LinkedList<FeldInfo> feldInfos,
				LinkedList<String> nichtBestimmbareFelder)
		{
			this._feldInfos = feldInfos;
			this._nichtBestimmbareFelder = nichtBestimmbareFelder;
		}

		public String getFeldText()
		{
			String re = "";

			Iterator<FeldInfo> it = _feldInfos.iterator();
			while (it.hasNext())
			{
				FeldInfo feldInfo = it.next();

				if (re.length() > 0)
				{
					re = re + " ";
				}

				re = re + feldInfo.getFeldDarstellungsString();

			}

			return re;
		}

		public LinkedList<FeldInfo> getFeldInfos()
		{
			return _feldInfos;
		}

		public LinkedList<String> getNichtBestimmbareFelder()
		{
			return _nichtBestimmbareFelder;
		}

		/**
		 * Liefert die sonstigen Informationen zu der Feldvalidierung
		 * 
		 * @return Sonstige Infos zur Feldvalidierung
		 */
		public String getSonstigeInfos()
		{
			return _sonstigeInfos;
		}

		/**
		 * Setzt die sonstigen Informationen zur Feldvalidierung.
		 * 
		 * @param sonstigeInfos
		 *          sonstige Informationen zur Feldvalidierung.
		 */
		protected void setSonstigeInfos(String sonstigeInfos)
		{
			this._sonstigeInfos = sonstigeInfos;
		}
	}

	public class FeldInfo
	{
		/**
		 * Wert des Feldes
		 */
		private int _wert = -1;

		/**
		 * Zeigt an, dass die Kurzform verwendet werden soll.
		 */
		private boolean _kurzform = false;

		/**
		 * Typ des Feldes. Wird an Hand der {@link ZeitKlasse} festgelegt.
		 */
		private ZeitKlasse _typ = null;

		public FeldInfo(ZeitKlasse typ, boolean kurzform, int wert)
		{
			this._typ = typ;
			this._kurzform = kurzform;
			this._wert = wert;
		}

		/**
		 * Liefert den Wert des Feldes.
		 * 
		 * @return Wert des Feldes
		 */
		public int getWert()
		{
			return _wert;
		}

		/**
		 * Liefert den Typ des Feldes.
		 * 
		 * @return Typ des Feldes als {@link ZeitKlasse}
		 */
		public ZeitKlasse getTyp()
		{
			return _typ;
		}

		/**
		 * Passt den Wert des Feldes in der Weise an, dass Min und Max Werte der
		 * {@link ZeitKlasse} nicht unter- oder berschritten werden.
		 */
		public void passeWertAn()
		{
			if (_wert < _typ.getBeschraenkungMin())
			{
				_wert = _typ._beschraenkungMin;
			} else if (_wert > _typ.getBeschraenkungMax())
			{
				_wert = _typ.getBeschraenkungMax();
			}
		}

		public String getFeldDarstellungsString()
		{
			String re = "";

			/*
			 * Wert setzen
			 */
			re = _wert + " ";

			if (_kurzform)
			{
				re = re + _typ.getKurzform();
			} else
			{
				if (_wert > 1)
				{
					re = re + _typ.getMehrzahl();
				} else
				{
					re = re + _typ.getBezeichnung();
				}
			}

			return re;
		}

		public String toString()
		{
			String re = "===FeldInfo===\n";
			re = re + _typ.toString() + "\n";
			re = re + "Wert: " + _wert + "\n";
			re = re + "==============\n";
			return re;
		}

	}

	public class SelektierInfo
	{
		private int _startPos = -1;
		private int _endPos = -1;
		private String _neuerText = null;

		public SelektierInfo(int startPos, int endPos, String neuerText)
		{
			this._startPos = startPos;
			this._endPos = endPos;
			this._neuerText = neuerText;
		}

		public int getStartPos()
		{
			return _startPos;
		}

		public int getEndPos()
		{
			return _endPos;
		}

		public String getNeuerText()
		{
			return _neuerText;
		}
		
		public String toString()
		{
			String re = "StartPos: " + _startPos + " EndPos: " + _endPos + " Text: " + _neuerText;
			return re;
		}

	}

	public class ZeitKlasse
	{
		/**
		 * Bezeichnung der {@link ZeitKlasse}
		 */
		private String _bezeichnung = null;

		/**
		 * Mehrzahl der Bezeichnung
		 */
		private String _mehrzahl = null;

		/**
		 * Kurzform der Bezeichnung dieser {@link ZeitKlasse}
		 */
		private String _kurzform = null;

		/**
		 * Beschrnkung im Wertebereich nach unten
		 */
		private int _beschraenkungMin = -1;

		/**
		 * Beschrnkung im Wertebereich nach oben
		 */
		private int _beschraenkungMax = -1;

		/**
		 * Konstruktor. bernimmt Bezeichnung und Kurzform dieser {@link ZeitKlasse}
		 * 
		 * @param bezeichnung
		 *          Bezeichnung
		 * @param mehrzahl
		 *          Mehrzahl der Bezeichnung
		 * @param kurzform
		 *          Kurzform der Bezeichnung
		 */
		public ZeitKlasse(String bezeichnung, String mehrzahl, String kurzform)
		{
			this._bezeichnung = bezeichnung;
			this._mehrzahl = mehrzahl;
			this._kurzform = kurzform;
		}

		/**
		 * Liefert die Bezeichnung dieser {@link ZeitKlasse}.
		 * 
		 * @return Bezeichnung
		 */
		public String getBezeichnung()
		{
			return _bezeichnung;
		}

		/**
		 * Mehrzahl der Bezeichnung dieser {@link ZeitKlasse}.
		 * 
		 * @return Mehrzahl der Bezeichnung
		 */
		public String getMehrzahl()
		{
			return _mehrzahl;
		}

		/**
		 * Liefert die Kurzform dieser {@link ZeitKlasse}.
		 * 
		 * @return Kurzform
		 */
		public String getKurzform()
		{
			return _kurzform;
		}

		public boolean istBeschraenkt()
		{
			boolean re = false;

			if ((_beschraenkungMin >= 0) && (_beschraenkungMax >= 0))
			{
				re = true;
			}

			return re;
		}

		/**
		 * Liefert die untere Grenze fr den Wert dieser Zeitklasse.
		 * 
		 * @return Untere Grenze. <code>-1</code> bedeutet, keine untere Grenze.
		 */
		public int getBeschraenkungMin()
		{
			return _beschraenkungMin;
		}

		public void setBeschraenkungMin(int beschraenkungMin)
		{
			this._beschraenkungMin = beschraenkungMin;
		}

		/**
		 * Liefert die obere Grenze fr den Wert dieser Zeitklasse.
		 * 
		 * @return Obere Grenze. <code>-1</code> bedeutet, keine obere Grenze.
		 */
		public int getBeschraenkungMax()
		{
			return _beschraenkungMax;
		}

		public void setBeschraenkungMax(int wert)
		{
			this._beschraenkungMax = wert;
		}

		/**
		 * Prft, ob ein Wert innerhalb der Beschrnkung liegt.
		 * 
		 * @param wert
		 *          Zu prfender Wert.
		 * @return Liefert <code>true</code>, wenn der Wert innerhalb der
		 *         Beschrnkung liegt, sonst <code>false</code>.
		 */
		public boolean verifiziereWert(int wert)
		{
			boolean re = true;

			if (!(_beschraenkungMin < 0) && !(_beschraenkungMax < 0))
			{

				if ((_beschraenkungMin >= 0) && (_beschraenkungMax >= 0))
				{
					if ((wert >= _beschraenkungMin) && (wert <= _beschraenkungMax))
					{
						re = true;
					} else
					{
						re = false;
					}
				}
			}

			return re;
		}

		/**
		 * Liefert einen alternativen Wert fr einen im gegebenen Wertebereich nicht
		 * zu verifizierenden Wert. Ist der Wert kleiner als das Minimum, wird das
		 * Minimum zurckgeliefert. Ist der Wert grer als das Maximum, wird das
		 * Maximum zurckgeliefert.
		 * 
		 * @param wert
		 *          Anzupassender Wert.
		 * @return Angepasster Wert, entsprechend dem Minimum oder dem Maximum. Muss
		 *         keine Anpassung durchgefhrt werden weil der Wert im Bereich
		 *         liegt, wird der Wert zurckgeliefert.
		 */
		public int getAlternativWert(int wert)
		{
			int re = wert;

			if (!(_beschraenkungMin < 0) && !(_beschraenkungMax < 0))
			{
				if (wert < _beschraenkungMin)
				{
					re = _beschraenkungMin;
				} else if (wert > _beschraenkungMax)
				{
					re = _beschraenkungMax;
				}
			}

			return re;
		}

		public String toString()
		{
			String re = "ZeitKlasse - Bezeichnung: " + _bezeichnung + " Kurzform: "
					+ _kurzform;
			return re;
		}

		/**
		 * Prft an Hand eines Strings, ob es sich um die Bezeichnung dieser
		 * {@link ZeitKlasse} handelt.
		 * 
		 * @param pruefString
		 *          Zu prfender String
		 * @return Liefert <code>true</code>, wenn es sich um die Bezeichnung
		 *         dieser {@link ZeitKlasse} handelt, sonst <code>false</code>.
		 */
		public boolean istStringBezeichnung(String pruefString)
		{
			boolean re = true;

			if (!_bezeichnung.equalsIgnoreCase(pruefString))
			{
				re = false;
			}

			return re;
		}

		/**
		 * Prft an Hand eines Strings, ob es sich um die Mehrzahl dieser
		 * {@link ZeitKlasse} handelt.
		 * 
		 * @param pruefString
		 *          Zu prfender String
		 * @return Liefert <code>true</code>, wenn es sich um die Mehrzahl dieser
		 *         {@link ZeitKlasse} handelt, sonst <code>false</code>.
		 */
		public boolean istStringMehrzahl(String pruefString)
		{
			boolean re = true;

			if (!_mehrzahl.equalsIgnoreCase(pruefString))
			{
				re = false;
			}

			return re;
		}

		/**
		 * Prft an Hand eines Strings, ob es sich um die Kurzform dieser
		 * {@link ZeitKlasse} handelt.
		 * 
		 * @param pruefString
		 *          Zu prfender String
		 * @return Liefert <code>true</code>, wenn es sich um die Kurzform dieser
		 *         {@link ZeitKlasse} handelt, sonst <code>false</code>.
		 */
		public boolean istStringKurzform(String pruefString)
		{
			boolean re = true;

			if (!_kurzform.equals(pruefString))
			{
				re = false;
			}

			return re;
		}

		public String getFeldDarstellungsString(int wert)
		{
			String re = "";

			/*
			 * Wert setzen
			 */
			re = wert + " ";

			if (_kurzdarstellung)
			{
				re = re + this.getKurzform();
			} else
			{
				if (wert > 1)
				{
					re = re + this.getMehrzahl();
				} else
				{
					re = re + this.getBezeichnung();
				}
			}

			return re;
		}

	}

	/**
	 * Liefert die {@link ZeitKlasse} Jahr.
	 * 
	 * @return Jahr
	 */
	public ZeitKlasse getZeitKlasseJahr()
	{
		return _jahr;
	}

	/**
	 * Liefert die {@link ZeitKlasse} Monat.
	 * 
	 * @return Monat
	 */
	public ZeitKlasse getZeitKlasseMonat()
	{
		return _monat;
	}

	/**
	 * Liefert die {@link ZeitKlasse} Tag.
	 * 
	 * @return Tag
	 */
	public ZeitKlasse getZeitKlasseTag()
	{
		return _tag;
	}

	/**
	 * Liefert die {@link ZeitKlasse} Stunde.
	 * 
	 * @return Stunde
	 */
	public ZeitKlasse getZeitKlasseStunde()
	{
		return _stunde;
	}

	/**
	 * Liefert die {@link ZeitKlasse} Minute.
	 * 
	 * @return Minute
	 */
	public ZeitKlasse getZeitKlasseMinute()
	{
		return _minute;
	}

	/**
	 * Liefert die {@link ZeitKlasse} Sekunde.
	 * 
	 * @return Sekunde
	 */
	public ZeitKlasse getZeitKlasseSekunde()
	{
		return _sekunde;
	}

	/**
	 * Liefert die {@link ZeitKlasse} Tausendstel.
	 * 
	 * @return Tausendstel.
	 */
	public ZeitKlasse getZeitKlasseTausendstel()
	{
		return _tausendstel;
	}

	@SuppressWarnings("unused")
	private String zeigeStringPositionen(String wert)
	{
		String re = null;

		if ((wert != null) && (wert.length() > 0))
		{
			re = "";
			for (int i = 0; i < wert.length(); i++)
			{
				re = re + i + ":=> " + wert.charAt(i) + "\n";
			}
		}

		return re;
	}

	public void setAusgangsZeitPunkt(long ausgangsZeitpunkt)
	{
		this._ausgangsZeitpunkt = ausgangsZeitpunkt;
	}

	public long getAusgangszeitpunkt()
	{
		return _ausgangsZeitpunkt;
	}

	public void setEndZeitpunkt(long endZeitPunkt)
	{
		this._endZeitPunkt = endZeitPunkt;
	}

	public long getEndZeitpunkt()
	{
		return _endZeitPunkt;
	}

	/**
	 * Berechnet auf Grund der gesetzten Felder jahre, monate, tage, stunden,
	 * minuten, sekunden und millisekunden die Zeitdauer
	 * 
	 * @return Zeitdauer in Millisekunden
	 */
	public long getZeitdauerInBezugAufAnfangszeitpunkt(String textFeld)
	{
		long zeitDauer = -1;

		int millisekunden = 0;
		int sekunden = 0;
		int minuten = 0;
		int stunden = 0;
		int tage = 0;
		int monate = 0;
		int jahre = 0;

		ValidierungsInfo felder = validiereFelder(textFeld);

		if (felder != null)
		{
			LinkedList<FeldInfo> feldInfos = felder.getFeldInfos();
			Iterator<FeldInfo> it = feldInfos.iterator();
			while (it.hasNext())
			{
				FeldInfo info = it.next();

				ZeitKlasse zeitKlasse = info.getTyp();

				if (zeitKlasse.equals(_tausendstel))
				{
					millisekunden = info.getWert();
				} else if (zeitKlasse.equals(_sekunde))
				{
					sekunden = info.getWert();
				} else if (zeitKlasse.equals(_minute))
				{
					minuten = info.getWert();
				} else if (zeitKlasse.equals(_stunde))
				{
					stunden = info.getWert();
				} else if (zeitKlasse.equals(_tag))
				{
					tage = info.getWert();
				} else if (zeitKlasse.equals(_monat))
				{
					monate = info.getWert();
				} else if (zeitKlasse.equals(_jahr))
				{
					jahre = info.getWert();
				}
			}

			GregorianCalendar gcAnfang = new GregorianCalendar();
			gcAnfang.setTimeInMillis(_ausgangsZeitpunkt);

			GregorianCalendar gcEnde = (GregorianCalendar) gcAnfang.clone();

			gcEnde.add(Calendar.MILLISECOND, millisekunden);
			gcEnde.add(Calendar.SECOND, sekunden);
			gcEnde.add(Calendar.MINUTE, minuten);
			gcEnde.add(Calendar.HOUR_OF_DAY, stunden);
			gcEnde.add(Calendar.DAY_OF_MONTH, tage);
			gcEnde.add(Calendar.MONTH, monate);
			gcEnde.add(Calendar.YEAR, jahre);

			zeitDauer = gcEnde.getTimeInMillis() - gcAnfang.getTimeInMillis();
		}

		return zeitDauer;
	}

	/**
	 * Berechnet die Basis einer Zeitdauer in Abhngigkeit von einem
	 * Startzeitpunkt
	 * 
	 * @param startZeitpunkt
	 *          Startzeitpunkt, auf dessen Basis die Zeitdauer in JMThmst bestimmt
	 *          wird.
	 * @param zeitDauer
	 *          Zeitdauer in Millisekunden
	 * @return String fr das Textfeld
	 * 
	 */
	public String setZeitdauerInBezugAufAnfangszeitpunkt(long startZeitpunkt,
			long zeitDauer)
	{
		long endZeitpunkt = startZeitpunkt + zeitDauer;

		String re = setZeidauerInBezugAufEndpunkt(zeitDauer, endZeitpunkt);
		return re;
	}

	/**
	 * Berechnet die Basis einer Zeitdauer in Abhngigkeit von einem Endzeitpunkt
	 * den Anfangszeitpunkt und die zwischen Anfangszeitpunkt und Endzeitpunkt
	 * liegenden Jahre, Monate, Tage, Stunden, Minuten, Sekunden und
	 * Millisekunden. Schaltjahre und unterschiedliche Monatslngen werden
	 * bercksichtigt.
	 * 
	 * @param zeitDauer
	 *          Zeitdauer in Millisekunden.
	 * @param endZeitpunkt
	 *          Endzeitpunkt, von dem Anfangszeitpunkt und JTMhmst bis zum Ende
	 *          berechnet werden sollen.
	 * @return String fr das Textfeld
	 */
	public String setZeidauerInBezugAufEndpunkt(long zeitDauer, long endZeitpunkt)
	{
		String re = "";

		long zeitEnde = endZeitpunkt;
		long zeitAnfang = zeitEnde - zeitDauer;

		int jahreWert = 0;
		int monateWert = 0;
		int tageWert = 0;
		int stundenWert = 0;
		int minutenWert = 0;
		int sekundenWert = 0;
		int tausendstelWert = 0;

		GregorianCalendar gcEnde = new GregorianCalendar();
		gcEnde.setTimeInMillis(zeitEnde);

		GregorianCalendar gcAnfangWirdzuEnde = new GregorianCalendar();
		gcAnfangWirdzuEnde.setTimeInMillis(zeitAnfang);

		int zaehlerJahre = 0;
		boolean jahre = true;

		while (jahre)
		{
			GregorianCalendar tempGcAnfang = (GregorianCalendar) gcAnfangWirdzuEnde
					.clone();
			tempGcAnfang.add(Calendar.YEAR, zaehlerJahre + 1);

			if (tempGcAnfang.getTimeInMillis() <= gcEnde.getTimeInMillis())
			{
				zaehlerJahre++;

			} else
			{
				jahre = false;
			}
		}
		gcAnfangWirdzuEnde.add(Calendar.YEAR, zaehlerJahre);

		int zaehlerMonate = 0;
		boolean monate = true;
		while (monate)
		{
			GregorianCalendar tempGcAnfang = (GregorianCalendar) gcAnfangWirdzuEnde
					.clone();
			tempGcAnfang.add(Calendar.MONTH, zaehlerMonate + 1);
			if (tempGcAnfang.getTimeInMillis() <= gcEnde.getTimeInMillis())
			{
				zaehlerMonate++;

			} else
			{
				monate = false;
			}
		}
		gcAnfangWirdzuEnde.add(Calendar.MONTH, zaehlerMonate);

		int zaehlerTage = 0;
		boolean tage = true;
		while (tage)
		{
			GregorianCalendar tempGcAnfang = (GregorianCalendar) gcAnfangWirdzuEnde
					.clone();
			tempGcAnfang.add(Calendar.DAY_OF_MONTH, zaehlerTage + 1);
			if (tempGcAnfang.getTimeInMillis() <= gcEnde.getTimeInMillis())
			{
				zaehlerTage++;

			} else
			{
				tage = false;
			}
		}
		gcAnfangWirdzuEnde.add(Calendar.DAY_OF_MONTH, zaehlerTage);

		int zaehlerStunden = 0;
		boolean stunden = true;
		while (stunden)
		{
			GregorianCalendar tempGcAnfang = (GregorianCalendar) gcAnfangWirdzuEnde
					.clone();
			tempGcAnfang.add(Calendar.HOUR_OF_DAY, zaehlerStunden + 1);
			if (tempGcAnfang.getTimeInMillis() <= gcEnde.getTimeInMillis())
			{
				zaehlerStunden++;
			} else
			{
				stunden = false;
			}
		}
		gcAnfangWirdzuEnde.add(Calendar.HOUR_OF_DAY, zaehlerStunden);

		int zaehlerMinuten = 0;
		boolean minuten = true;
		while (minuten)
		{
			GregorianCalendar tempGcAnfang = (GregorianCalendar) gcAnfangWirdzuEnde
					.clone();
			tempGcAnfang.add(Calendar.MINUTE, zaehlerMinuten + 1);
			if (tempGcAnfang.getTimeInMillis() <= gcEnde.getTimeInMillis())
			{
				zaehlerMinuten++;
			} else
			{
				minuten = false;
			}
		}
		gcAnfangWirdzuEnde.add(Calendar.MINUTE, zaehlerMinuten);

		int zaehlerSekunden = 0;
		boolean sekunden = true;
		while (sekunden)
		{
			GregorianCalendar tempGcAnfang = (GregorianCalendar) gcAnfangWirdzuEnde
					.clone();
			tempGcAnfang.add(Calendar.SECOND, zaehlerSekunden + 1);
			if (tempGcAnfang.getTimeInMillis() <= gcEnde.getTimeInMillis())
			{
				zaehlerSekunden++;
			} else
			{
				sekunden = false;
			}
		}
		gcAnfangWirdzuEnde.add(Calendar.SECOND, zaehlerSekunden);

		int zaehlerTausendstel = 0;
		boolean millisekunden = true;
		while (millisekunden)
		{
			GregorianCalendar tempGcAnfang = (GregorianCalendar) gcAnfangWirdzuEnde
					.clone();
			tempGcAnfang.add(Calendar.MILLISECOND, zaehlerTausendstel + 1);
			if (tempGcAnfang.getTimeInMillis() <= gcEnde.getTimeInMillis())
			{
				zaehlerTausendstel++;
			} else
			{
				millisekunden = false;
			}
		}
		gcAnfangWirdzuEnde.add(Calendar.MILLISECOND, zaehlerTausendstel);

		// System.out.println("GcAnfangWirdzuEnde: " +
		// gcAnfangWirdzuEnde.toString());

		jahreWert = zaehlerJahre;
		monateWert = zaehlerMonate;
		tageWert = zaehlerTage;
		stundenWert = zaehlerStunden;
		minutenWert = zaehlerMinuten;
		sekundenWert = zaehlerSekunden;
		tausendstelWert = zaehlerTausendstel;

		boolean verJahr = _jahr.verifiziereWert(jahreWert);
		boolean verMonat = _monat.verifiziereWert(monateWert);
		boolean verTag = _tag.verifiziereWert(tageWert);
		boolean verStunde = _stunde.verifiziereWert(stundenWert);
		boolean verMinute = _minute.verifiziereWert(minutenWert);
		boolean verSekunde = _sekunde.verifiziereWert(sekundenWert);
		boolean verTausendstel = _tausendstel.verifiziereWert(tausendstelWert);

		if ((!verJahr) || (!verMonat) || (!verTag) || (!verStunde) || (!verMinute)
				|| (!verSekunde) || (!verTausendstel))
		{
			String meldung = "Einer der Werte liegt nicht im gltigen Bereich:\n\n";

			if (!verJahr)
			{
				meldung = meldung + " Jahr = " + jahreWert + " (Min: "
						+ _jahr.getBeschraenkungMin() + " Max: "
						+ _jahr.getBeschraenkungMax() + ")\n";
			} else if (!verMonat)
			{
				meldung = meldung + " Monat = " + monateWert + " (Min: "
						+ _monat.getBeschraenkungMin() + ") (Max: "
						+ _monat.getBeschraenkungMax() + ")\n";
			} else if (!verTag)
			{
				meldung = meldung + " Tag = " + tageWert + " (Min: "
						+ _tag.getBeschraenkungMin() + " Max: "
						+ _tag.getBeschraenkungMax() + ")\n";
			} else if (!verStunde)
			{
				meldung = meldung + " Stunde = " + stundenWert + " (Min: "
						+ _stunde.getBeschraenkungMin() + " Max: "
						+ _stunde.getBeschraenkungMax() + ")\n";
			} else if (!verMinute)
			{
				meldung = meldung + " Minute = " + minutenWert + " (Min: "
						+ _minute.getBeschraenkungMin() + " Max: "
						+ _minute.getBeschraenkungMax() + ")\n";
			} else if (!verSekunde)
			{
				meldung = meldung + " Sekunde = " + sekundenWert + " (Min: "
						+ _sekunde.getBeschraenkungMin() + " Max: "
						+ _sekunde.getBeschraenkungMax() + ")\n";
			} else if (!verTausendstel)
			{
				meldung = meldung + " Tausendstel = " + tausendstelWert + " (Min: "
						+ _tausendstel.getBeschraenkungMin() + " Max: "
						+ _tausendstel.getBeschraenkungMax() + ")\n";
			}

			meldung = meldung + "\n";
			meldung = meldung
					+ "Soll eine automatische Anpassung der Werte an den gltigen Bereich erfolgen?";

			MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.ICON_QUESTION | SWT.OK | SWT.CANCEL);
			mb.setText("Normaliesierung: Werte nicht im gltigen Bereich!");
			mb.setMessage(meldung);

			int open = mb.open();
			if (open == SWT.OK)
			{
				jahreWert = _jahr.getAlternativWert(jahreWert);
				monateWert = _monat.getAlternativWert(monateWert);
				tageWert = _tag.getAlternativWert(tageWert);
				stundenWert = _stunde.getAlternativWert(stundenWert);
				minutenWert = _minute.getAlternativWert(minutenWert);
				sekundenWert = _sekunde.getAlternativWert(sekundenWert);
				tausendstelWert = _tausendstel.getAlternativWert(tausendstelWert);
			}

		}

		re = baueZeitString(jahreWert, monateWert, tageWert, stundenWert,
				minutenWert, sekundenWert, tausendstelWert);

		return re;
	}

	/**
	 * Normalisiert die Zeitdauer in Bezug auf den Anfangszeitpunkt an Hand des
	 * {@link GregorianCalendar}. So werden z.B. auf 61 Sekunden 1 Minute und 2
	 * Sekunden.
	 * 
	 * @param textFeld
	 *          Inhalt der Textfeldes, welches normalisiert werden soll.
	 * @return Normalisierter Inhalt fr das Textfeld.s
	 */
	public String normalisieren(String textFeld)
	{
		String re = setZeitdauerInBezugAufAnfangszeitpunkt(_ausgangsZeitpunkt,
				getZeitdauerInBezugAufAnfangszeitpunkt(textFeld));
		return re;
	}

	/**
	 * Baut einen Zeitstring fr das Textfeld zusammen. Die Parameter mssen >= 0
	 * sein, bei < 0 wird das entsprechende Feld nicht dargestellt.
	 * 
	 * @param jahre
	 *          Anzahl der Jahre.
	 * @param monate
	 *          Anzahl der Monate
	 * @param tage
	 *          Anzahl der Tage
	 * @param stunden
	 *          Anzahl der Stunden
	 * @param minuten
	 *          Anzahl der Minuten
	 * @param sekunden
	 *          Anzahl der Sekunden
	 * @param tausendstel
	 *          Anzahl der Tausendstel
	 * @return String fr das Textfeld.
	 */
	public String baueZeitString(int jahre, int monate, int tage, int stunden,
			int minuten, int sekunden, int tausendstel)
	{
		String re = "";

		if (_jahreDarstellen || jahre > 0)
		{
			if (jahre > -1)
			{
				re = re + jahre + " ";

				if (_kurzdarstellung)
				{
					re = re + _jahr.getKurzform();
				} else
				{
					if (jahre == 0 || jahre > 1)
					{
						re = re + _jahr.getMehrzahl();
					} else
					{
						re = re + _jahr.getBezeichnung();
					}
				}

				re = re + " ";
			}
		}

		if (_monateDarstellen || monate > 0)
		{
			if (monate > -1)
			{
				re = re + monate + " ";

				if (_kurzdarstellung)
				{
					re = re + _monat.getKurzform();
				} else
				{
					if (monate == 0 || monate > 1)
					{
						re = re + _monat.getMehrzahl();
					} else
					{
						re = re + _monat.getBezeichnung();
					}
				}

				re = re + " ";
			}
		}

		if (_tageDarstellen || tage > 0)
		{
			if (tage > -1)
			{
				re = re + tage + " ";

				if (_kurzdarstellung)
				{
					re = re + _tag.getKurzform();
				} else
				{
					if (tage == 0 || tage > 1)
					{
						re = re + _tag.getMehrzahl();
					} else
					{
						re = re + _tag.getBezeichnung();
					}
				}

				re = re + " ";
			}
		}

		if (_stundenDarstellen || stunden > 0)
		{
			if (stunden > -1)
			{
				re = re + stunden + " ";

				if (_kurzdarstellung)
				{
					re = re + _stunde.getKurzform();
				} else
				{
					if (stunden == 0 || stunden > 1)
					{
						re = re + _stunde.getMehrzahl();
					} else
					{
						re = re + _stunde.getBezeichnung();
					}
				}

				re = re + " ";
			}
		}

		if (_minutenDarstellen || minuten > 0)
		{
			if (minuten > -1)
			{
				re = re + minuten + " ";

				if (_kurzdarstellung)
				{
					re = re + _minute.getKurzform();
				} else
				{
					if (minuten == 0 || minuten > 1)
					{
						re = re + _minute.getMehrzahl();
					} else
					{
						re = re + _minute.getBezeichnung();
					}
				}

				re = re + " ";
			}
		}

		if (_sekundenDarstellen || sekunden > 0)
		{
			if (sekunden > -1)
			{
				re = re + sekunden + " ";

				if (_kurzdarstellung)
				{
					re = re + _sekunde.getKurzform();
				} else
				{
					if (sekunden == 0 || sekunden > 1)
					{
						re = re + _sekunde.getMehrzahl();
					} else
					{
						re = re + _sekunde.getBezeichnung();
					}
				}

				re = re + " ";
			}
		}

		if (_tausendstelDarstellen || tausendstel > 0)
		{
			if (tausendstel > -1)
			{
				re = re + tausendstel + " ";

				if (_kurzdarstellung)
				{
					re = re + _tausendstel.getKurzform();
				} else
				{
					if (tausendstel == 0 || tausendstel > 1)
					{
						re = re + _tausendstel.getMehrzahl();
					} else
					{
						re = re + _tausendstel.getBezeichnung();
					}
				}

				re = re + " ";
			}
		}

		if ((re.length() - 1) >= 0)
		{
			re = re.substring(0, re.length() - 1);
		}

		return re;
	}

	/**
	 * Liefert die Anzahl der Jahre in einem String.
	 * 
	 * @param textFeld
	 *          String
	 * @return Anzahl der Jahre.
	 */
	protected int getJahre(String textFeld)
	{
		int re = 0;
		ValidierungsInfo validiereFelder = validiereFelder(textFeld);
		Iterator<FeldInfo> it = validiereFelder.getFeldInfos().iterator();
		while (it.hasNext())
		{
			FeldInfo feldInfo = it.next();
			if (feldInfo.getTyp().equals(_jahr))
			{
				re = feldInfo.getWert();
				break;
			}
		}

		return re;
	}

	protected String setJahre(String textFeld, int jahre)
	{
		String re = "";

		int monate = getMonate(textFeld);
		int tage = getTage(textFeld);
		int stunden = getStunden(textFeld);
		int minuten = getMinuten(textFeld);
		int sekunden = getSekunden(textFeld);
		int tausendstel = getTausendstel(textFeld);

		re = baueZeitString(jahre, monate, tage, stunden, minuten, sekunden,
				tausendstel);

		return re;
	}

	/**
	 * Liefert die Anzahl der Monate in einem String.
	 * 
	 * @param textFeld
	 *          String
	 * @return Anzahl der Monate.
	 */
	protected int getMonate(String textFeld)
	{
		int re = 0;
		ValidierungsInfo validiereFelder = validiereFelder(textFeld);
		Iterator<FeldInfo> it = validiereFelder.getFeldInfos().iterator();
		while (it.hasNext())
		{
			FeldInfo feldInfo = it.next();
			if (feldInfo.getTyp().equals(_monat))
			{
				re = feldInfo.getWert();
				break;
			}
		}

		return re;
	}

	protected String setMonate(String textFeld, int monate)
	{
		String re = "";

		int jahre = getJahre(textFeld);
		int tage = getTage(textFeld);
		int stunden = getStunden(textFeld);
		int minuten = getMinuten(textFeld);
		int sekunden = getSekunden(textFeld);
		int tausendstel = getTausendstel(textFeld);

		re = baueZeitString(jahre, monate, tage, stunden, minuten, sekunden,
				tausendstel);

		return re;
	}

	/**
	 * Liefert die Anzahl der Tage in einem String.
	 * 
	 * @param textFeld
	 *          String
	 * @return Anzahl der Tage.
	 */
	protected int getTage(String textFeld)
	{
		int re = 0;
		ValidierungsInfo validiereFelder = validiereFelder(textFeld);
		Iterator<FeldInfo> it = validiereFelder.getFeldInfos().iterator();
		while (it.hasNext())
		{
			FeldInfo feldInfo = it.next();
			if (feldInfo.getTyp().equals(_tag))
			{
				re = feldInfo.getWert();
				break;
			}
		}

		return re;
	}

	protected String setTage(String textFeld, int tage)
	{
		String re = "";

		int jahre = getJahre(textFeld);
		int monate = getMonate(textFeld);
		int stunden = getStunden(textFeld);
		int minuten = getMinuten(textFeld);
		int sekunden = getSekunden(textFeld);
		int tausendstel = getTausendstel(textFeld);

		re = baueZeitString(jahre, monate, tage, stunden, minuten, sekunden,
				tausendstel);

		return re;
	}

	/**
	 * Liefert die Anzahl der Stunden in einem String.
	 * 
	 * @param textFeld
	 *          String
	 * @return Anzahl der Stunden.
	 */
	protected int getStunden(String textFeld)
	{
		int re = 0;
		ValidierungsInfo validiereFelder = validiereFelder(textFeld);
		Iterator<FeldInfo> it = validiereFelder.getFeldInfos().iterator();
		while (it.hasNext())
		{
			FeldInfo feldInfo = it.next();
			if (feldInfo.getTyp().equals(_stunde))
			{
				re = feldInfo.getWert();
				break;
			}
		}

		return re;
	}

	protected String setStunden(String textFeld, int stunden)
	{
		String re = "";

		int jahre = getJahre(textFeld);
		int monate = getMonate(textFeld);
		int tage = getTage(textFeld);
		int minuten = getMinuten(textFeld);
		int sekunden = getSekunden(textFeld);
		int tausendstel = getTausendstel(textFeld);

		re = baueZeitString(jahre, monate, tage, stunden, minuten, sekunden,
				tausendstel);

		return re;
	}

	/**
	 * Liefert die Anzahl der Minuten in einem String.
	 * 
	 * @param textFeld
	 *          String
	 * @return Anzahl der Minuten.
	 */
	protected int getMinuten(String textFeld)
	{
		int re = 0;
		ValidierungsInfo validiereFelder = validiereFelder(textFeld);
		Iterator<FeldInfo> it = validiereFelder.getFeldInfos().iterator();
		while (it.hasNext())
		{
			FeldInfo feldInfo = it.next();
			if (feldInfo.getTyp().equals(_minute))
			{
				re = feldInfo.getWert();
				break;
			}
		}

		return re;
	}

	protected String setMinuten(String textFeld, int minuten)
	{
		String re = "";

		int jahre = getJahre(textFeld);
		int monate = getMonate(textFeld);
		int tage = getTage(textFeld);
		int stunden = getStunden(textFeld);
		int sekunden = getSekunden(textFeld);
		int tausendstel = getTausendstel(textFeld);

		re = baueZeitString(jahre, monate, tage, stunden, minuten, sekunden,
				tausendstel);

		return re;
	}

	/**
	 * Liefert die Anzahl der Sekunden in einem String.
	 * 
	 * @param textFeld
	 *          String
	 * @return Anzahl der Sekunden.
	 */
	protected int getSekunden(String textFeld)
	{
		int re = 0;
		ValidierungsInfo validiereFelder = validiereFelder(textFeld);
		Iterator<FeldInfo> it = validiereFelder.getFeldInfos().iterator();
		while (it.hasNext())
		{
			FeldInfo feldInfo = it.next();
			if (feldInfo.getTyp().equals(_sekunde))
			{
				re = feldInfo.getWert();
				break;
			}
		}

		return re;
	}

	protected String setSekunden(String textFeld, int sekunden)
	{
		String re = "";

		int jahre = getJahre(textFeld);
		int monate = getMonate(textFeld);
		int tage = getTage(textFeld);
		int stunden = getStunden(textFeld);
		int minuten = getMinuten(textFeld);
		int tausendstel = getTausendstel(textFeld);

		re = baueZeitString(jahre, monate, tage, stunden, minuten, sekunden,
				tausendstel);

		return re;
	}

	/**
	 * Liefert die Anzahl der Tausendstel in einem String.
	 * 
	 * @param textFeld
	 *          String
	 * @return Anzahl der Tausendstel.
	 */
	protected int getTausendstel(String textFeld)
	{
		int re = 0;
		ValidierungsInfo validiereFelder = validiereFelder(textFeld);
		Iterator<FeldInfo> it = validiereFelder.getFeldInfos().iterator();
		while (it.hasNext())
		{
			FeldInfo feldInfo = it.next();
			if (feldInfo.getTyp().equals(_tausendstel))
			{
				re = feldInfo.getWert();
				break;
			}
		}

		return re;
	}

	protected String setTausendstel(String textFeld, int tausendstel)
	{
		String re = "";

		int jahre = getJahre(textFeld);
		int monate = getMonate(textFeld);
		int tage = getTage(textFeld);
		int stunden = getStunden(textFeld);
		int minuten = getMinuten(textFeld);
		int sekunden = getSekunden(textFeld);

		re = baueZeitString(jahre, monate, tage, stunden, minuten, sekunden,
				tausendstel);

		return re;
	}

	/**
	 * Bestimmt die {@link ZeitDauerGranularitaet} an Hand des Textfeldes.
	 * 
	 * @param textFeld
	 * @return {@link ZeitDauerGranularitaet}
	 */
	protected ZeitDauerGranularitaet getGranularitaet(String textFeld)
	{
		ZeitDauerGranularitaet re = null;

		ValidierungsInfo validiereFelder = validiereFelder(textFeld);

		if (validiereFelder.getFeldInfos() != null)
		{
			Iterator<FeldInfo> it = validiereFelder.getFeldInfos().iterator();
			while (it.hasNext())
			{
				FeldInfo feldInfo = it.next();
				if (re == null && feldInfo.getTyp().equals(_jahr))
				{
					re = ZeitDauerGranularitaet.jahr;
				} else if (re == null && feldInfo.getTyp().equals(_monat))
				{
					re = ZeitDauerGranularitaet.monat;
				} else if (re == null && feldInfo.getTyp().equals(_tag))
				{
					re = ZeitDauerGranularitaet.tag;
				} else if (re == null && feldInfo.getTyp().equals(_stunde))
				{
					re = ZeitDauerGranularitaet.stunde;
				} else if (re == null && feldInfo.getTyp().equals(_minute))
				{
					re = ZeitDauerGranularitaet.minute;
				} else if (re == null && feldInfo.getTyp().equals(_sekunde))
				{
					re = ZeitDauerGranularitaet.sekunde;
				} else if (re == null && feldInfo.getTyp().equals(_tausendstel))
				{
					re = ZeitDauerGranularitaet.tausendstel;
				}

			}
		}

		if (re == null)
		{
			re = ZeitDauerGranularitaet.jahr;
		}
		return re;
	}

}
