/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.streckenprofil.sts.a8.daten;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.concurrent.CopyOnWriteArrayList;

/**
 * Die Klasse kapselt die Verwaltungsmethoden fr die Listener-Funktionalitt 
 * der {@link IStreckenProfilDaten}. Diese Klasse implemntiert das Singleton-Pattern
 * und stellt dadurch sicher dass der Listener nur einmal Instanziiert wird. 
 * 
 * 
 * @author Pittner
 *
 */
public class StreckenProfilDataUpdateListener
{
  /**
   * Liste der {@link IStreckenProfilDataUpdateListener}, die ber einen neue DaV-Daten informiert werden mchten.
   */
  private CopyOnWriteArrayList<IStreckenProfilDataUpdateListener> _streckenProfilDataUpdateListener = new CopyOnWriteArrayList<IStreckenProfilDataUpdateListener>();
  
  /**
   * Singleton-Instanz
   */
  private static StreckenProfilDataUpdateListener _instanz = null;

  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  public static StreckenProfilDataUpdateListener getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new StreckenProfilDataUpdateListener();
    }
    return _instanz;
  }
    

  /**
   * Fgt einen Listener zur Liste der DaVDataUpdate-Listener hinzu.
   * 
   * @param listener
   */
  public void addDaVDataUpdateListener(IStreckenProfilDataUpdateListener listener)
  {
    if (!_streckenProfilDataUpdateListener.contains(listener))
    {
      _streckenProfilDataUpdateListener.add(listener);
    }
    else
    {
      System.err
          .println("Der Listener ist bereits in der Zuhrerliste vorhanden und wird nicht erneut hinzugefgt");
    }
  }
  
  /**
   * Benachrichtigt alle registrierten {@link IStreckenProfilDataUpdateListener} darber, dass neue Daten vorhanden sind.
   * 
   */
  public void notifyStreckenProfilDataUpdateListener(IStreckenProfilDaten daten)
  {
//    System.out.println("notifyStreckenProfilDataUpdateListener(): " + daten.getClass().getName());

    StreckenProfilDataUpdateEvent event = new StreckenProfilDataUpdateEvent(this, daten);

    Iterator<IStreckenProfilDataUpdateListener> it = _streckenProfilDataUpdateListener.iterator();
    while (it.hasNext())
    {
      IStreckenProfilDataUpdateListener listener = it.next();
      listener.streckenProfilDataUpdate(event);
    }
  }
  
  /**
   * Entfernt einen {@link IStreckenProfilDataUpdateListener} von der Liste der Zuhrer.
   * 
   * @param IStreckenProfilDataUpdateListener
   *          Zu entfernender {@link IStreckenProfilDataUpdateListener}.
   */
  public void removeDaVDataUpdateListener(IStreckenProfilDataUpdateListener listener)
  {
//    System.out.println("StreckenProfilDataUpdateListener.removeDaVDataUpdateListener()");
    
    if (_streckenProfilDataUpdateListener.contains(listener))
    {
      _streckenProfilDataUpdateListener.remove(listener);
    }
    else
    {
      System.err.println("Der Listener ist in der Zuhrerliste nicht vorhanden und wird somit nicht entfernt.");
    }
  }
  
}
