/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.dynobj.configurationAreas;

import java.io.IOException;
import java.io.StringReader;
import java.net.URL;
import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceStore;
import org.jdom.Attribute;
import org.jdom.DocType;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import de.bsvrz.sba.plugin.dynobj.Activator;

/**
 * Controller fr das {@link ConfigurationAreaModel}. Speichert/Liest die
 * Zuordnungen Konfigurationsbereich zu Objekttyp
 * <ul>
 * <li>im/aus PreferenceStore,</li>
 * <li>in/aus einer CSV-Datei oder</li>
 * <li>in/aus einer XML-Datei</li>
 * </ul>
 * 
 * @author HofmannJ
 * 
 */
public class ConfigurationAreaControler
{
	/**
	 * Modell der ConfigurationArea
	 */
	private ConfigurationAreaModel _model = null;

	/**
	 * Konstruktor, arbeitet auf einem neuen, leeren
	 * {@link ConfigurationAreaModel}.
	 */
	public ConfigurationAreaControler()
	{
		this._model = new ConfigurationAreaModel();
	}

	/**
	 * Konstruktor, bernimmt ein bereits vorhandenes
	 * {@link ConfigurationAreaModel}.
	 * 
	 * @param model
	 *          Zu bernehmendes {@link ConfigurationAreaModel}.
	 */
	public ConfigurationAreaControler(ConfigurationAreaModel model)
	{
		this._model = model;
	}

	/**
	 * Liefert das vom Controler verwendete {@link ConfigurationAreaModel} zurck.
	 * 
	 * @return Verwendetes {@link ConfigurationAreaModel}
	 */
	public ConfigurationAreaModel getModel()
	{
		return _model;
	}

	/**
	 * Liest das {@link ConfigurationAreaModel} aus einem {@link PreferenceStore}
	 * ein. Das Dirty-Flag wird resettet.
	 * 
	 * @param store
	 *          {@link PreferenceStore}, aus dem das
	 *          {@link ConfigurationAreaModel} eingelesen werden soll.
	 */
	public void readFromPreferenceStore(IPreferenceStore store)
	{
		_model.clear();
		
		int count = store.getInt(DefsPrefStore.P_COUNT);
		if (count > 0)
		{
			for (int i = 0; i < count; i++)
			{
				String value = store.getString(DefsPrefStore.P_ELEMENT_PREFIX + i);
				String[] split = value.split(";");

				if (split.length > 1)
				{
					String pidObjTyp = split[0];
					String pidConfArea = split[1];

					if ((pidObjTyp.length() > 0) && (pidConfArea.length() > 0))
					{
						_model.add(pidObjTyp, pidConfArea);
					}

				}
			}
			_model.resetDirtyFlag();
		}

	}

	/**
	 * Schreibt das {@link ConfigurationAreaModel} in einen
	 * {@link PreferenceStore}. Setzt <b>nicht</b> das Dirty-Flag zurck!
	 * 
	 * @param store
	 *          {@link PreferenceStore}, in den das
	 *          {@link ConfigurationAreaModel} geschrieben werden soll.
	 */
	public void writeToPreferenceStore(IPreferenceStore store)
	{
		LinkedList<ConfigurationAreaModelRecord> structure = _model.getStructure();

		store.setValue(DefsPrefStore.P_COUNT, structure.size());

		Iterator<ConfigurationAreaModelRecord> it = structure.iterator();
		int i = 0;
		while (it.hasNext())
		{
			ConfigurationAreaModelRecord entry = it.next();
			store.setValue(DefsPrefStore.P_ELEMENT_PREFIX + i, entry
					.getPidObjectType()
					+ ";" + entry.getPidConfigurationArea());
			i++;
		}

	}

	/**
	 * Liest das {@link ConfigurationAreaModel} aus einem CSV-String ein. Das
	 * Dirty-Flag wird resettet.
	 * 
	 * @param csv
	 *          CSV-String, aus dem das {@link ConfigurationAreaModel} eingelesen
	 *          werden soll.
	 * @throws IllegalArgumentException
	 *           Wird geworfen, wenn der CSV-String <code>null</code> ist oder
	 *           die Lnge 0 hat.
	 */
	public void readFromCsvString(String csv) throws IllegalArgumentException
	{
		_model.clear();
		
		if ((csv != null) && (csv.length() > 0))
		{
			String[] splitLines = csv.split("\n");
			for (String line : splitLines)
			{
				String[] splitEntries = line.split(";");
				_model.add(splitEntries[0], splitEntries[1]);
			}
			_model.resetDirtyFlag();
		}
		else
		{
			throw (new IllegalArgumentException(
					"Einlesen aus CSV-String nicht mglich. Der String ist null oder hat die Lnge 0."));
		}

	}

	/**
	 * Schreibt das {@link ConfigurationAreaModel} in einen CSV-String. Setzt
	 * <b>nicht</b> das Dirty-Flag zurck!
	 * 
	 * @return Aus dem {@link ConfigurationAreaModel} erzeugter CSV-String. Ist
	 *         <code>null</code>, wenn das Modell keine Daten enthlt.
	 */
	public String writeToCsvString()
	{
		String re = null;

		// LinkedList<String[]> structure = _model.getStructureOld();
		LinkedList<ConfigurationAreaModelRecord> structure = _model.getStructure();

		if (structure.size() > 0)
		{
			StringBuffer sb = new StringBuffer();

			Iterator<ConfigurationAreaModelRecord> it = structure.iterator();
			while (it.hasNext())
			{
				ConfigurationAreaModelRecord entry = it.next();

				if (entry != null)
				{
					String pidObjectType = entry.getPidObjectType();
					String pidConfigurationArea = entry.getPidConfigurationArea();
					if ((pidObjectType != null) && (pidConfigurationArea != null))
					{
						sb.append(pidObjectType + ";" + pidConfigurationArea + "\n");
					}
				}

			}
			re = sb.toString();
		}

		return re;
	}

	/**
	 * Liest das {@link ConfigurationAreaModel} aus einen XML-String. Das
	 * Dirty-Flag wird resettet.
	 * 
	 * @param xmlString
	 *          XML-String, aus dem das {@link ConfigurationAreaModel} gelesen
	 *          werden soll.
	 * @throws IllegalArgumentException
	 *           Wird geworfen, wenn das XML-Dokument nicht den Erwartungen
	 *           entspricht.
	 */
	public void readFromXmlString(String xmlString)
			throws IllegalArgumentException
	{
		_model.clear();
		
		SAXBuilder builder = new SAXBuilder();

		builder.setValidation(true);

		builder.setEntityResolver(new EntityResolver()
		{
			public InputSource resolveEntity(String publicId, String systemId)
					throws SAXException, IOException
			{
				if ((systemId != null) && (systemId.endsWith(DefsXml.xmlDocSystemId)))
				{
					URL url = Activator.getDefault().getBundle().getResource(
							"xml/" + DefsXml.xmlDocSystemId);
					return new InputSource(url.toExternalForm());
				}

				return null;
			}
		});

		StringReader reader = new StringReader(xmlString);
		Document doc;
		try
		{
			doc = builder.build(reader);

			Element elementWurzel = doc.getRootElement();

			if (elementWurzel.getName().equals(DefsXml.XML_ELEMENT_BASIS))
			{
				@SuppressWarnings("unchecked")
				Iterator it = elementWurzel.getChildren(DefsXml.XML_ELEMENT_ZUORDNUNG)
						.iterator();
				while (it.hasNext())
				{
					Object object = it.next();
					if (object instanceof Element)
					{
						Element element = (Element) object;
						if (element.getName().equals(DefsXml.XML_ELEMENT_ZUORDNUNG))
						{
							String pidObjectType = element
									.getAttributeValue(DefsXml.XML_ATT_OBJEKTTYP);
							String pidConfigurationArea = element
									.getAttributeValue(DefsXml.XML_ATT_KONFIGURATIONSBEREICH);

							if ((pidObjectType != null) && (pidConfigurationArea != null))
							{
								_model.add(pidObjectType, pidConfigurationArea);
							}
						}
					}
				}
				_model.resetDirtyFlag();
			}
			else
			{
				throw (new IllegalArgumentException(
						"Das Wurzelelement ist nicht vom Typ " + DefsXml.XML_ELEMENT_BASIS
								+ ", sondern vom Typ " + elementWurzel.getName()
								+ ". Breche den Vorgang ab."));
			}

		}
		catch (JDOMException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}

	/**
	 * Schreibt das {@link ConfigurationAreaModel} in einen XML-String. Setzt
	 * <b>nicht</b> das Dirty-Flag zurck!
	 * 
	 * @return Aus dem {@link ConfigurationAreaModel} erzeugter XML-String.
	 */
	public String writeToXmlString()
	{
		String re = null;

		Element root = new Element(DefsXml.XML_ELEMENT_BASIS);
		Document doc = new Document(root);

		Iterator<ConfigurationAreaModelRecord> it = _model.getStructure()
				.iterator();
		while (it.hasNext())
		{
			ConfigurationAreaModelRecord record = it.next();
			Element ele = new Element(DefsXml.XML_ELEMENT_ZUORDNUNG);
			Attribute attPidObjTyp = new Attribute(DefsXml.XML_ATT_OBJEKTTYP, record
					.getPidObjectType());
			ele.setAttribute(attPidObjTyp);
			Attribute attPidConfArea = new Attribute(
					DefsXml.XML_ATT_KONFIGURATIONSBEREICH, record
							.getPidConfigurationArea());
			ele.setAttribute(attPidConfArea);
			root.addContent(ele);
		}

		/*
		 * DocType
		 */
		// if (false)
		// {
		DocType doctype = new DocType(DefsXml.xmlDocTypeElementName,
				DefsXml.xmlDocTypePublicId, DefsXml.xmlDocSystemId);
		doc.setDocType(doctype);
		// }

		/*
		 * Formatierung und Ausgabe
		 */
		Format prettyFormat = Format.getPrettyFormat();
		XMLOutputter out = new XMLOutputter(prettyFormat);
		Format f = out.getFormat();
		f.setEncoding(DefsXml.xmlCodierung);
		out.setFormat(f);
		re = out.outputString(doc);

		return re;

	}

}
