/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.dynobj.configurationAreas;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.TreeMap;
import java.util.Map.Entry;

import org.eclipse.jface.preference.PreferenceStore;

/**
 * Speichert das Modell der Zuordnungen ObjectTyp zu ConfigurationArea. Die
 * {@link TreeMap} wird ber Synchronized-Blcke gesichert, diese Klasse ist
 * damit Threadsicher.<br>
 * <br>
 * <b>Hinweis:</b> Abfragen des im {@link PreferenceStore} hinterlegten Modells
 * sind ber die Klasse {@link ConfigurationAreaDirectAccessPreferenceStore}
 * mglich.
 * 
 * @author HofmannJ
 * 
 */
public class ConfigurationAreaModel
{
	/**
	 * Speichert die Zuordnung ObjectType zu ConfigurationArea Aufbau: Pid
	 * ObjectType - Pid ConfigurationArea
	 */
	private TreeMap<String, String> _hmConfigurationAreas = null;

	/**
	 * Dirty-Flag
	 */
	private boolean _isDirty = false;

	/**
	 * Konstruktor.
	 */
	public ConfigurationAreaModel()
	{
		_hmConfigurationAreas = new TreeMap<String, String>();
	}

	/**
	 * Fgt eine neue Zuordnung von ObjectType zu ConfigurationArea zur
	 * {@link TreeMap} hinzu. Diese Methode ist Threadsicher. Das Dirty-Flag wird
	 * gesetzt.
	 * 
	 * @param pidObjectType
	 *          Pid des ObjectTypes, dient auch als Key in der {@link TreeMap}.
	 *          Ein vorhandener Eintrag unter dem gleichen Key wird berschrieben!
	 * @param pidConfigurationArea
	 *          Pid des ConfigurationAreas.
	 */
	public void add(String pidObjectType, String pidConfigurationArea)
	{
		synchronized (_hmConfigurationAreas)
		{
			_hmConfigurationAreas.put(pidObjectType, pidConfigurationArea);
			_isDirty = true;
		}
	}

	/**
	 * Liefert zu der Pid eines ObjectTypes die Pid der zugehrigen
	 * ConfigurationArea. Diese Methode ist Threadsicher,.
	 * 
	 * @param pidObjectType
	 *          Pid des ObjectTypes
	 * @return Pid der zugehrigen ConfigurationArea oder <code>null</code>,
	 *         wenn zu dem ObjectType keine Zuordnung registriert ist.
	 */
	public String getPidConfigurationArea(String pidObjectType)
	{
		String re = null;

		if (_hmConfigurationAreas.containsKey(pidObjectType))
		{
			re = _hmConfigurationAreas.get(pidObjectType);
		}

		return re;
	}

	/**
	 * Setzt zu einem ObjectType eine neue ConfigurationArea. Verwendet die
	 * Methode {@link ConfigurationAreaModel#add(String, String)}, diese ist
	 * Threadsicher.
	 * 
	 * @param pidObjectType
	 *          Pid des ObjectTypes, fr welchen eine neu Zuordnung zu einer
	 *          ConfigurationArea getroffen werden soll.
	 * @param pidConfigurationArea
	 *          Pid der ConfigurationArea, welche dem ObjectType zugeordnet sein
	 *          soll.
	 */
	public void setPidConfigurationArea(String pidObjectType,
			String pidConfigurationArea)
	{
		add(pidObjectType, pidConfigurationArea);
	}

	/**
	 * Entfernt eine Zuordnung von ObjectType zu ConfigurationArea an Hand der Pid
	 * des ObjectTyps als Key aus der {@link TreeMap}, sofern diese Zuordnung
	 * existent ist. Diese Methode ist Threadsicher. Das Dirty-Flag wird gesetzt.
	 * 
	 * @param pidObjectType
	 *          Pid des ObjectTypes, dessen Zuordnung zu einem ConfigurationArea
	 *          entfernt werden soll.
	 */
	public void remove(String pidObjectType)
	{
		synchronized (_hmConfigurationAreas)
		{
			if (_hmConfigurationAreas.containsKey(pidObjectType))
			{
				_hmConfigurationAreas.remove(pidObjectType);
				_isDirty = true;
			}
		}
	}

	/**
	 * Liefert eine Liste der vorhandenen Zuordnungen von ObjectType zu
	 * ConfigurationArea in einer Liste vom Typ
	 * {@link ConfigurationAreaModelRecord}. Diese Methode ist Threadsicher.
	 * 
	 * @return Liste der vorhandenen Zuordnungen von ObjectType zu
	 *         ConfigurationArea
	 */
	public LinkedList<ConfigurationAreaModelRecord> getStructure()
	{
		LinkedList<ConfigurationAreaModelRecord> re = new LinkedList<ConfigurationAreaModelRecord>();

		synchronized (_hmConfigurationAreas)
		{
			Iterator<Entry<String, String>> it = _hmConfigurationAreas.entrySet()
					.iterator();
			while (it.hasNext())
			{
				Entry<String, String> entry = it.next();
				re.add(new ConfigurationAreaModelRecord(entry.getKey(), entry
						.getValue()));
			}
		}

		return re;
	}

	/**
	 * Leert die {@link TreeMap}. Diese Methode ist Threadsicher. Das Dirty-Flag
	 * wird gesetzt.
	 */
	public void clear()
	{
		synchronized (_hmConfigurationAreas)
		{
			_hmConfigurationAreas.clear();
			_isDirty = true;
		}
	}

	/**
	 * Setzt das Dirty-Flag auf False zurck.
	 */
	public void resetDirtyFlag()
	{
		_isDirty = false;
	}

	/**
	 * Prft, ob dieses Modell "dirty" ist. Dies ist der Fall, wenn neue Daten
	 * hinzugefgt oder Daten gelscht worden sind.
	 * 
	 * @return
	 */
	public boolean isDirty()
	{
		return _isDirty;
	}

	public String toString()
	{
		StringBuffer sb = new StringBuffer();

		LinkedList<ConfigurationAreaModelRecord> structure = getStructure();
		sb.append("Anzahl Eintrge: " + structure.size() + "\n");

		Iterator<ConfigurationAreaModelRecord> it = structure.iterator();
		int i = 0;
		while (it.hasNext())
		{
			ConfigurationAreaModelRecord modelRecord = it.next();
			sb.append("Eintrag #" + i + ": " + modelRecord.toString() + "\n");
			i++;
		}

		return sb.toString();
	}

}
