/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.navigator.navigatorModell;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.navigator.navigatorModell.INavigatorElement.STATUS;

/**
 * Klasse der Elemente des Navigators
 * 
 * @author Krause
 *
 */
public class NavigatorElement implements INavigatorElement
{ 
  
  /**
   * Flag zum Debuggen
   */
  protected static final boolean m_debug = false;
  
  /**
   * mgliche Kategorien des NavigatorElements, welches den
   * Bildarten des NavigatorElements entspricht, die zur Verfgung stehen
   * - falls eine neue Kategorie (Bildart) hinzu kommt, muss der entsprechende Filename
   *   bis zum "_" hier eingetragen werden -
   */
  public static enum KATEGORIE {unbekannt, 
    Wetterstation, KonfigBereich, Inselbus, Unterzentrale, 
    Streckenstation, EinAusKonz, KommRechnerIF, Datenendgeraet, 
    DatenendgeraetDe128, DatenendgeraetDeAxl, DatenendgeraetDeLve, 
    DatenendgeraetDeSys, DatenendgeraetDeUfd, DatenendgeraetDeUfdNW, 
    DatenendgeraetDeVlt, DatenendgeraetDeWww, DatenendgeraetDeWzg,
    Anzeigequerschnitte, Anzeigequerschnitt, Anzeige, 
    Messquerschnitte, Messquerschnitt, Unterspannvers, Fahrspur,
    Ordner, Kalender, Ereignis, Ereignistyp, UrsachenEinheit, UmfeldDatenMessStelle, UmfeldDatenSensor,
    UmfeldDatenSensorHelligkeit, UmfeldDatenSensorFahrBahnOberflaechenTemperatur,
    UmfeldDatenSensorFahrBahnOberflaechenZustand, UmfeldDatenSensorGefrierTemperatur,
    UmfeldDatenSensorLuftTemperatur, UmfeldDatenSensorNiederschlagsart,
    UmfeldDatenSensorNiederschlagsintensitaet, UmfeldDatenSensorRelativeLuftFeuchte,
    UmfeldDatenSensorRestSalz, UmfeldDatenSensorSichtWeite, UmfeldDatenSensorTaupunktTemperatur,
    UmfeldDatenSensorTemperaturInTiefe1, UmfeldDatenSensorTemperaturInTiefe3,
    UmfeldDatenSensorWasserFilmDicke};

  /**
   * Anzahl der maximal mglichen Stati zu den Bildarten (Elementtypen)
   */
  private static final HashMap<KATEGORIE, Integer> ANZAHLSTATI = new HashMap<KATEGORIE, Integer>();
  static
  {
    ANZAHLSTATI.put( KATEGORIE.Datenendgeraet,          6 );
    ANZAHLSTATI.put( KATEGORIE.DatenendgeraetDe128,     6 );
    ANZAHLSTATI.put( KATEGORIE.DatenendgeraetDeAxl,     6 );
    ANZAHLSTATI.put( KATEGORIE.DatenendgeraetDeLve,     6 );
    ANZAHLSTATI.put( KATEGORIE.DatenendgeraetDeSys,     6 );
    ANZAHLSTATI.put( KATEGORIE.DatenendgeraetDeUfd,     6 );
    ANZAHLSTATI.put( KATEGORIE.DatenendgeraetDeUfdNW,   6 );
    ANZAHLSTATI.put( KATEGORIE.DatenendgeraetDeVlt,     6 );
    ANZAHLSTATI.put( KATEGORIE.DatenendgeraetDeWww,     6 );
    ANZAHLSTATI.put( KATEGORIE.DatenendgeraetDeWzg,     6 );
    ANZAHLSTATI.put( KATEGORIE.EinAusKonz,              6 );
    ANZAHLSTATI.put( KATEGORIE.Inselbus,                6 );
    ANZAHLSTATI.put( KATEGORIE.KommRechnerIF,           6 );
    ANZAHLSTATI.put( KATEGORIE.KonfigBereich,           6 );
    ANZAHLSTATI.put( KATEGORIE.Streckenstation,         6 );
    ANZAHLSTATI.put( KATEGORIE.Unterzentrale,           6 );
    ANZAHLSTATI.put( KATEGORIE.Wetterstation,           6 );
    ANZAHLSTATI.put( KATEGORIE.Anzeigequerschnitt,      6 );
    ANZAHLSTATI.put( KATEGORIE.Anzeigequerschnitte,     6 );
    ANZAHLSTATI.put( KATEGORIE.Ereignis,                5 );
    ANZAHLSTATI.put( KATEGORIE.Anzeige,                 4 );
    ANZAHLSTATI.put( KATEGORIE.Messquerschnitt,         2 );
    ANZAHLSTATI.put( KATEGORIE.Fahrspur,                2 );
    ANZAHLSTATI.put( KATEGORIE.Messquerschnitte,        2 );
    ANZAHLSTATI.put( KATEGORIE.Unterspannvers,          2 );
    ANZAHLSTATI.put( KATEGORIE.Ordner,                  0 );
    ANZAHLSTATI.put( KATEGORIE.Kalender,                0 );
    ANZAHLSTATI.put( KATEGORIE.Ereignistyp,             0 );
    ANZAHLSTATI.put( KATEGORIE.unbekannt,               0 );
    ANZAHLSTATI.put( KATEGORIE.UmfeldDatenMessStelle,   0 );
    ANZAHLSTATI.put( KATEGORIE.UmfeldDatenSensor,       0 );
    ANZAHLSTATI.put( KATEGORIE.UmfeldDatenSensorHelligkeit, 0 );
    ANZAHLSTATI.put( KATEGORIE.UmfeldDatenSensorFahrBahnOberflaechenTemperatur, 0 );
    ANZAHLSTATI.put( KATEGORIE.UmfeldDatenSensorFahrBahnOberflaechenZustand, 0 );
    ANZAHLSTATI.put( KATEGORIE.UmfeldDatenSensorGefrierTemperatur, 0 );
    ANZAHLSTATI.put( KATEGORIE.UmfeldDatenSensorLuftTemperatur, 0 );
    ANZAHLSTATI.put( KATEGORIE.UmfeldDatenSensorNiederschlagsart, 0 );
    ANZAHLSTATI.put( KATEGORIE.UmfeldDatenSensorNiederschlagsintensitaet, 0 );
    ANZAHLSTATI.put( KATEGORIE.UmfeldDatenSensorRelativeLuftFeuchte, 0 );
    ANZAHLSTATI.put( KATEGORIE.UmfeldDatenSensorRestSalz, 0 );
    ANZAHLSTATI.put( KATEGORIE.UmfeldDatenSensorSichtWeite, 0 );
    ANZAHLSTATI.put( KATEGORIE.UmfeldDatenSensorTaupunktTemperatur, 0 );
    ANZAHLSTATI.put( KATEGORIE.UmfeldDatenSensorTemperaturInTiefe1, 0 );
    ANZAHLSTATI.put( KATEGORIE.UmfeldDatenSensorTemperaturInTiefe3, 0 );
    ANZAHLSTATI.put( KATEGORIE.UmfeldDatenSensorWasserFilmDicke, 0 );
  
  }
  
  /**
   * Ordner in dem sich die Bilder befinden
   */  
  protected static final String IMAGEORDNER = "icons/items/";
  
  /**
   * mgliche Bildstati fr die Bildart
   * - enthlt zu den Stati die entsprechenden Endungen der Imagefiles inklusive "_"
   * - maximal mglich sind: "_unb.png", "_IO.png", "_tIO.png", "_tSt.png", "_TeilSt.png", "_St.png"
   * - Sonderstati fr Ereignisse: "_unb.png", "_ungueltig.png", "_zeit.png", "_verkehr.png", "_zeitverkehr.png"
   */
  protected final HashMap<STATUS,String> m_imagestatus;
  
  /**
   * Kategorie des NavigatorElements
   * entspricht der Bildart, welche das Bild kennzeichnet 
   */
  protected KATEGORIE  m_kategorie;

  /**
   * Flag, ob die Kinder ausgeklappt sind
   */
  protected boolean m_ausgeklappt;

  /**
   * Status (unbekannt, Strung, usw)
   */
  protected STATUS m_status;

  /**
   * Name, der im Baum angezeigt wird
   */
  protected String m_name;

  /**
   * Bildpfad zum Bild, das im Baum angezeigt wird
   */
  private String m_imagePath;
  
  /**
   * Elternteil, das bergeordnet ist (auch Wurzel genannt)
   */
  protected Object m_parent;
  
  /**
   * zugeordnete SystemObjekte
   */
  protected LinkedList<SystemObject> m_listeObjekte;
  
  /**
   * Liste der direkten Kinder (untergeordnete Elemente)
   */
  protected List<Object> m_childrenList;
  
  /**
   * Listener fr nderungen im DatenModell
   */
  protected IDeltaListener listener = NullDeltaListener.getInstance();

  public NavigatorElement()
  {
	this.m_imagestatus = new HashMap<STATUS, String>();
  }
  
  //--------------------------------------------------------------------- 
  /**
   * Konstruktor fr ein Element des Navigators
   * 
   * @param parent
   *          Elternteil
   * @param name
   *          angezeigter Name
   * @param kategorie
   *          Kategorie des NavigatorElements, entspricht dem Teil des Filenames
   *          bis zum "_"
   * @param listeObjekte
   *          Liste der zugeordneten Systemobjekte
   */
  public NavigatorElement(Object parent, String name, KATEGORIE kategorie, LinkedList<SystemObject> listeObjekte)
  {
    this.m_parent = parent;
    this.m_name = name;
    this.m_ausgeklappt = false;
    this.m_childrenList = new ArrayList<Object>();
    if ( listeObjekte == null )
      this.m_listeObjekte = new LinkedList<SystemObject>();
    else
      this.m_listeObjekte = listeObjekte;
    
    if ( parent != null )
    {
      ((INavigatorElement) parent).addChild( this );
    }
    
    int anzahlStati = 0;
    if ( kategorie != null )
    {
      this.m_kategorie = kategorie;
      anzahlStati = ANZAHLSTATI.get( m_kategorie );
    }

    this.m_imagestatus = new HashMap<STATUS, String>();
    
    //HashMap der Bildstati mit den Endungen der Imagefiles fllen
    if (m_imagestatus.size() == 0 && anzahlStati == 5)
    {
      //Sonderfall bei Ereignissen
      this.m_imagestatus.put( STATUS.unbekannt, "_unb.png" );
      this.m_imagestatus.put( STATUS.ungueltig, "_ungueltig.png" );
      this.m_imagestatus.put( STATUS.zeitlich_gueltig, "_zeit.png" );
      this.m_imagestatus.put( STATUS.verkehrlich_gueltig, "_verkehr.png" );
      this.m_imagestatus.put( STATUS.zeitverkehrlich_gueltig, "_zeitverkehr.png" );
      
    }
    else if ( m_imagestatus.size() == 0 && anzahlStati > 0 )
    {
      this.m_imagestatus.put( STATUS.unbekannt, "_unb.png" );
      this.m_imagestatus.put( STATUS.IO, "_IO.png" );
      if ( anzahlStati >= 4 )
      {
        this.m_imagestatus.put( STATUS.TeilStoerung, "_TeilSt.png" );
        this.m_imagestatus.put( STATUS.Stoerung, "_St.png" );

        if ( anzahlStati == 6 )
        {
          this.m_imagestatus.put( STATUS.teilweiseIO, "_tIO.png" );
          this.m_imagestatus.put( STATUS.teilweiseStoerung, "_tSt.png" );
        }
      }
    }

    // Status "unbekannt" vorbelegen
    if ( name != null )
      setStatus( STATUS.unbekannt );
    
  }
    
  @Override
  public Object[] getChildren()
  {
    return m_childrenList.toArray();
  }

//  @Override
//  public Image getImage()
//  {
//    return m_image;
//  }

  @Override
  public String getImagePath()
  {
    return m_imagePath;
  }
  
  @Override
  public Object getParent()
  {
    return m_parent;
  }

  @Override
  public LinkedList<SystemObject> getObjekte()
  {
    return m_listeObjekte;
  }
  
  @Override
  public STATUS getStatus()
  {
    return m_status;
  }

  @Override
  public String getName()
  {
    return m_name;
  }
  
  @Override
  public boolean hasObjekt(SystemObject objekt)
  {
    boolean enthalten = false;
    for (Iterator iterator = m_listeObjekte.iterator(); iterator.hasNext();)
    {
      SystemObject systemobjekt = (SystemObject) iterator.next();
      if ( systemobjekt.equals( objekt ) )
      {
        enthalten = true;
        break;
      }   
    }
    return enthalten;  

  }
  
  @Override
  public boolean isAusgeklappt()
  {
    return m_ausgeklappt;
  }

  @Override
  public void setAusgeklappt( boolean b )
  {
    m_ausgeklappt = b;
  }

  @Override
  public void setChildren( ArrayList<Object> children )
  {
    m_childrenList = children;
  }

  @Override
  public void addChild( INavigatorElement child )
  {
    m_childrenList.add( child );
  }

  @Override
  public void setImagePath( String bildPfad )
  {
    m_imagePath = bildPfad;
  }
  
  public void setName( String name )
  {
    m_name = name;
  }
  
  @Override
  public void setParent( INavigatorElement parent )
  {
    m_parent = parent;
  }


  @Override
  public boolean hasChildren()
  {
    // Rckgabe, ob das Element Kinder hat
    return m_childrenList == null ? false : m_childrenList.size() > 0;
  }

  @Override
  public boolean hasParent()
  {
    // Rckgabe, ob das Element einen Elternteil hat
    return m_parent == null ? false : true;
  }
  
  /**
   * Setzt den Status und das entsprechende Bild
   * sowie den Status des Elternteils und die Aktualisierung des Teilbaums
   * 
   * @param status    
   *          Status des NavigatorElementes
   * @param IMAGEORDNER 
   *          Ordner in dem sich die Images befinden
   * @param m_kategorie 
   *          Kategorie des NavigatorElements also Bildart, welche verwendet werden soll         
   */
  @Override
  public boolean setStatus( STATUS status )
  {

    //Bild zum Status entsprechend setzen
    String imagefile = null;
    
    if ( status != null && m_imagestatus.size()>0 )
      imagefile = m_imagestatus.get( status );
    
    if ( imagefile != null && m_kategorie != null )
      imagefile = IMAGEORDNER + m_kategorie.name() + imagefile;
    else if ( m_kategorie != null )
      imagefile = IMAGEORDNER + m_kategorie.name() + ".png";
    else
      imagefile = null;
    
    if ( m_debug )
      System.out.println(imagefile);

    if (imagefile != null && !imagefile.equals( "" ))
      setImagePath( imagefile );
        
    //Status setzen, Bild zum Elternteil neu setzen und TeilBaum aktualisieren
    if ( status != null )
    {
      if (m_debug)
        System.out.println("Setze neuen Status: " + status);
      
      STATUS statusalt = getStatus();
      m_status = status;

      //Status und Bild zum Elternteil neu setzen
      setParentStatus(status, statusalt);
      
      //TeilBaum aktualisieren
      if ( statusalt != null )
        upd( this );
      
    }
    
    return true;
    
  }

  /**
   * Status und Bild zum Elternteil neu setzen
   * 
   * @param status			neuer Status des Element
   * @param statusalt		alter Status des Element
   */
	private void setParentStatus(STATUS status, STATUS statusalt)
	{
		if (m_debug)
		{
		  System.out.println("Werte fr Autostatus: ");
		  System.out.println("Statusalt: " + statusalt + " Statusneu: " + status);
		}
		INavigatorElement parent = (INavigatorElement) getParent();
		
		if ( parent != null && status != null && !status.equals( statusalt ) )
		{
		  //Beim Start unntige Statusermittlungen des Elternteils unterlassen
//        if (!( statusalt == null && status.equals( STATUS.unbekannt ) && parent.getStatus().equals( STATUS.unbekannt ) ))
//        {
		    if (m_debug)
		      System.out.println( "AutoStatus setzen bei " + parent.getName() );
		    parent.setAutoStatus();
//        }
		}
	}

  @Override
  public boolean setAutoStatus()
  {
    if ( m_kategorie == null || ANZAHLSTATI.get( m_kategorie ) == 0 )
      return false;
    
    STATUS status = null;
    STATUS statusneu = null;

    Object[] children = getChildren();
    for (int i = 0; i < children.length; i++)
    {
      INavigatorElement child = (INavigatorElement) children[i];
      statusneu = child.getStatus();

      if (m_debug)
        System.out.println( "Aktueller Status: " + status + " Status von Child: " + statusneu );

      if ( statusneu == null )
      {
        continue;
      }

      if ( status == null )
      {
        status = statusneu;
        continue;
      }

      switch (status)
        {
        case unbekannt:
        {
          switch (statusneu)
            {
            case IO:
            case teilweiseIO:
              if ( ANZAHLSTATI.get( m_kategorie ) == 6 )
                status = STATUS.teilweiseIO;
              else
                status = STATUS.IO;
              break;
            case Stoerung:
            case teilweiseStoerung:
              if ( ANZAHLSTATI.get( m_kategorie ) == 6 )
                status = STATUS.teilweiseStoerung;
              else
                status = STATUS.Stoerung;                
              break;
            case TeilStoerung:
              if ( ANZAHLSTATI.get( m_kategorie ) >= 4 )
                status = STATUS.TeilStoerung;
              else
                status = STATUS.unbekannt;
              break;
            }
          break;
        }
        case IO:
        {
          switch (statusneu)
            {
            case unbekannt:
            case teilweiseIO:
              if ( ANZAHLSTATI.get( m_kategorie ) == 6 )
                status = STATUS.teilweiseIO;
              else
                status = STATUS.IO;
              break;
            case Stoerung:
            case teilweiseStoerung:
            case TeilStoerung:
              if ( ANZAHLSTATI.get( m_kategorie ) >= 4 )
                status = STATUS.TeilStoerung;
              else
                status = STATUS.unbekannt;
              break;
            }
          break;
        }
        case Stoerung:
        {
          switch (statusneu)
            {
            case unbekannt:
            case teilweiseStoerung:
              if ( ANZAHLSTATI.get( m_kategorie ) == 6 )
                status = STATUS.teilweiseStoerung;
              else
                status = STATUS.Stoerung;  
              break;
            case IO:
            case teilweiseIO:
            case TeilStoerung:
              if ( ANZAHLSTATI.get( m_kategorie ) >= 4 )
                status = STATUS.TeilStoerung;
              else
                status = STATUS.unbekannt;
              break;
            }
          break;
        }
        case teilweiseIO:
        {
          switch (statusneu)
            {
            case Stoerung:
            case teilweiseStoerung:
            case TeilStoerung:
              if ( ANZAHLSTATI.get( m_kategorie ) >= 4 )
                status = STATUS.TeilStoerung;
              else
                status = STATUS.unbekannt;
              break;
            }
          break;
        }
        case teilweiseStoerung:
        {
          switch (statusneu)
            {
            case IO:
            case teilweiseIO:
            case TeilStoerung:
              if ( ANZAHLSTATI.get( m_kategorie ) >= 4 )
                status = STATUS.TeilStoerung;
              else
                status = STATUS.unbekannt;
              break;
            }
          break;
        }
        }//switch status
    }

    if ( status == null )
      return false;
    else
    {
      setStatus( status );
      return true;
    }
  }

  /**
   * Liefert die Kategorie (auch Bildart)
   */
  public KATEGORIE getKategorie()
  {
    return m_kategorie;
  }
  
  /**
   * Neues Element als Beispiel ermitteln
   */
  public static INavigatorElement newElement()
  {
    INavigatorElement neu = new NavigatorElement(null, "neues Element", KATEGORIE.unbekannt, null);
    return neu;
  }
  
//  @Override
//  public LinkedList<Object> getKontextMenue()
//  {
//    NavigatorElementMitStati parent = (NavigatorElementMitStati)getParent();
//    KontextMenuInhalt kontextMenu = new KontextMenuInhalt(m_kategorie, parent.getKategorie()  ,getObjekte());  
//    return kontextMenu.getKontextListe();
//
//  }
    
  //------------------------------------------------------------
  // Methoden fr nderungen im DatenModell
  //------------------------------------------------------------

  /**
   * Listener setzen
   */
  public void addListener(IDeltaListener listener) 
  {
    this.listener = listener;
  }
  
  /**
   * Listener entfernen
   */  
  public void removeListener(IDeltaListener listener) 
  {
    if (m_debug)
      System.out.println("Remove Listener from: " + this.getName());
    if(this.listener.equals(listener))
    {
      if (m_debug)
        System.out.println("Remove erfolgreich: Listener gleich!");
      this.listener = NullDeltaListener.getInstance();
    }
  }
  
  /**
   * Kind zum Element gendert
   */
  public void upd(INavigatorElement toUpd)
  {
    listener.update(new DeltaEvent(toUpd));
  }
  
  /**
   * Kind zum Element hinzufgen
   */ 
  public void add(INavigatorElement toAdd)
  {
    if (m_debug)
      System.out.println("Element " + toAdd.getName() + " hinzufgen!");
    
    m_childrenList.add( toAdd );
    toAdd.setParent( this );
    
    // Event zum Listener hinzufgen, um den Baum zu aktualisieren
    listener.add( new DeltaEvent(toAdd) );
  }
  
  /**
   * Kind vom Element entfernen
   */
  public void remove(INavigatorElement toRemove)
  {
    if (m_debug)
      System.out.println("Element " + toRemove.getName() + " entfernen!");
    m_childrenList.remove( toRemove );
    toRemove.addListener( NullDeltaListener.getInstance() );
    
    // Event vom Listener entfernen, um den Baum zu aktualisieren
    listener.remove( new DeltaEvent(toRemove) );
  }
  

  
  //------------------------------------------------------------
  // Datenverteiler
  //------------------------------------------------------------
  /**
   * Status aus Datenverteiler ermitteln
   */
  @Override
  public void update( ResultData[] results )
  {
    
    
    for (ResultData resultData : results)
    {
//      System.out.println("NavigatorElement Daten fr: " + resultData);

      if (resultData.getDataDescription().getAttributeGroup().getPid().equals( "atg.tlsGloDeFehler" ))
      {
        if (resultData.hasData())
        {        
          String statusalt = resultData.getData().getScaledValue( "DEFehlerStatus" ).getValueText();
          
          if (m_debug)
            System.out.println("\nStatus: " + statusalt);
          
          if (statusalt.equals( "ok" ))
          {
            setStatus( STATUS.IO );
          }
          else if (statusalt.equals( "StrEAK" ))
          {
            setStatus( STATUS.Stoerung );
          }
          else if (statusalt.equals( "StrSM" ))
          {
            setStatus( STATUS.Stoerung );
          }
          else
          {
            setStatus( STATUS.unbekannt );
          }
        }
        else
        {
          setStatus( STATUS.unbekannt );
        }
      }//if tlsGloDeFehler   
      
      if (resultData.getDataDescription().getAttributeGroup().getPid().equals( "atg.anzeigeEigenschaftIst" ))
      {       
        if (resultData.hasData())
        {        
          String statusText = resultData.getData().getScaledValue( "Status" ).getValueText();
          
          if (m_debug)
            System.out.println("\nStatus: " + statusText);
          
          if (statusText.equals( "OK" ))
          {
            setStatus( STATUS.IO );
          }
          else if (statusText.equals( "Gestrt" ))
          {
            setStatus( STATUS.Stoerung );
          }
          else if (statusText.equals( "Kommunikationsausfall" ))
          {
            setStatus( STATUS.Stoerung );
          }
          else if (statusText.equals( "Teilstrung" ))
          {
            setStatus( STATUS.TeilStoerung );
          }
          else
          {
            setStatus( STATUS.unbekannt );
          }
        }
        else
        {
          setStatus( STATUS.unbekannt );
        }
      }//if anzeigeEigenschaftIst  
      
      if (resultData.getDataDescription().getAttributeGroup().getPid().equals( "atg.ereignis" ))
      {       
        if (resultData.hasData())
        {        
          boolean statusTextZeitlich = resultData.getData().getScaledValue( "zeitlichGltig" ).getValueText().equals( "Ja" );
          boolean statusTextVerkehrlich = resultData.getData().getScaledValue( "verkehrlichGltig" ).getValueText().equals( "Ja" );
          
          if (m_debug)
            System.out.println("\nStatusZeitlich: " + statusTextZeitlich + " StatusVerkehrlich: " + statusTextVerkehrlich);
          
          if ( statusTextZeitlich && statusTextVerkehrlich )
          {
              setStatus( STATUS.zeitverkehrlich_gueltig );
          }
          else if ( !statusTextZeitlich && statusTextVerkehrlich )
          {
              setStatus( STATUS.verkehrlich_gueltig );
          }
          else if ( statusTextZeitlich && !statusTextVerkehrlich )
          {
            setStatus( STATUS.zeitlich_gueltig );
          }
          else if ( !statusTextZeitlich && !statusTextVerkehrlich )
          {
            setStatus( STATUS.ungueltig );
          }
          else
          {
            setStatus( STATUS.unbekannt );
          }
        }
        else
        {
          setStatus( STATUS.unbekannt );
        }
      }//if ereignis 
      
      if (resultData.getDataDescription().getAttributeGroup().getPid().equals("atg.sswSbaEinzelsituationVorstufe"))
      {       
        if (resultData.hasData())
        {        
          boolean statusAktiv = resultData.getData().getScaledValue("Status").getValueText().equals("Ja");
          boolean statusAutomatik = resultData.getData().getScaledValue("StatusBercksichtigung").getValueText().equals("automatik");
          
          if (m_debug)
            System.out.println("\nAktiv: " + statusAktiv + " Automatik: " + statusAutomatik);
          
//          if ( statusAktiv && statusAutomatik )
//          {
//              setStatus( STATUS.zeitverkehrlich_gueltig );
//          }
//          else if ( !statusAktiv && statusAutomatik)
//          {
//              setStatus( STATUS.verkehrlich_gueltig );
//          }
//          else if ( statusAktiv && !statusAutomatik )
//          {
//            setStatus( STATUS.zeitlich_gueltig );
//          }
//          else if ( !statusAktiv && !statusAutomatik )
//          {
//            setStatus( STATUS.ungueltig );
//          }
//          else
//          {
//            setStatus( STATUS.unbekannt );
//          }

          if (statusAktiv)
          {
            setStatus(STATUS.verkehrlich_gueltig);
          }
          else if (!statusAktiv)
          {
            setStatus( STATUS.ungueltig );
          }
          else
          {
            setStatus( STATUS.unbekannt );
          }

        }
        else
        {
          setStatus( STATUS.unbekannt );
        }
      }//if atg.sswSbaEinzelsituationVorstufe 
      
    }//for
    
  }
  
}
