/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.dav;

import java.util.Date;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.ProjektSpezifischeEinstellungen;

/**
 * Klasse zum Erstellen der Attributgruppe <code>atg.wwaDeaktivierungAnzeigen</code> unter dem Aspekt <code>asp.parameterVorgabe</code> fr ein Objekt
 * vom Typ <code>typ.windWarnAnlage</code> und Senden dieser Atrributgruppe an den DaV. 
 * @author Dambach Werke GmbH
 */
public class SendeParameter implements ClientSenderInterface
{
  /**
   * Debug Anmelden am DaV
   */
  private boolean _debugAnmelde = true;
  
  /**
   * Debug Ausgaben
   */
  private boolean _debug = true;
  
  /**
   * Verbindung zum DaV
   */
  private ClientDavInterface _connection;
  
  /**
   * Definition der Attributgruppe
   */
  private String _atg = "";
  
  /**
   * bentigter Aspekt
   */
  private String _asp = "asp.parameterVorgabe";

  /**
   * Rolle
   */
  private SenderRole _role = SenderRole.sender();
    
  /**
   * Pid der SBA
   */
  private String _pidSBA = null;

  /**
   * Merker ob Empfnger bereit ist
   */
  private boolean _empfaengerBereit = false;

  /**
   * Privater Konstruktor der Klasse
   */
  public SendeParameter (String pidAtg)
  {
    this._connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    _pidSBA = ProjektSpezifischeEinstellungen.getPidSba();
    
    _atg = pidAtg;
  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum
   * Senden der Attibutgruppe atgPid unter dem Aspekt aspPid. 
   * @param objPid Pid des Objekts 
   * @param atgPid Pid der Attributgruppe
   * @param aspPid Pid des Aspekts
   */

  public DataDescription anmeldeSendeDaV (String objPid, String atgPid, String aspPid, SenderRole role)
  {
    if (objPid == null || atgPid == null || aspPid == null || _role == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    if (_debugAnmelde)
      System.out.println("Anmelden am DaV (Senden): " + objPid + " " + atgPid + "  " + aspPid + " " + role.toString());
    
    SystemObject sysObj = _connection.getDataModel().getObject(  objPid); 

    try
    {
      _connection.subscribeSender( this, sysObj, dd, role);
    }
    catch ( OneSubscriptionPerSendData e1 )
    {
      e1.printStackTrace();
    }
    
    return dd;
  }

  /**
   * Methode die das Objekt objPid beim Datenverteiler abmeldet zum
   * Senden der Attibutgruppe atgPid unter dem Aspekt aspPid.
   * @param objPid Pid des Objekts 
   * @param atgPid Attributgruppe die abgemeldet werden soll
   * @param aspPid Apekt der abgemeldet werden soll
   */
  public void abmeldeSendeDaV (String objPid, String atgPid, String aspPid)
  {
    if (objPid == null || atgPid == null || aspPid == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    if (_debugAnmelde)
      System.out.println("Abmelden am DaV (Senden): " + objPid + " " + atgPid + "  " + aspPid);

    SystemObject obj = _connection.getDataModel().getObject(objPid);
    
    _connection.unsubscribeSender(this, obj, dd);
  }


  /**
   * Methode zum Senden der Attributgruppe an den DaV
   */
   
  public synchronized void sendeParameter (Data data)
  {
    this._connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    if (_connection == null)
    {
      System.err.println(this.getClass() + " Keine Verbindung zum DaV");
      return;
    }

    anmeldeSendeDaV( _pidSBA, _atg, _asp, _role );

    // Prfen ob Empfnger vorhanden ist
    
    if (!_empfaengerBereit)
    {
      // u.U. beim ersten Mal warten
      
      for (int i=0; i<5 && !_empfaengerBereit; ++i)
      {
        try
        {
          Thread.sleep(1000);
        } 
        catch (InterruptedException e)
        {
        }
      }
    }
    
    if (!_empfaengerBereit)
    {
      System.err.println(this.getClass() + " Keine Empfnger fr Datensatz vorhanden !");
      
      generiereMeldungFehler( "Parameter Windrichtung", 
	                            "Daten konnten nicht an den Steuerrechner bertragen werden,\n" +
	                            "da kein Empfnger fr die Attributgruppe am DaV angemeldet ist !\n" +
	                            "Bitte kontaktieren sie ihren Systemadministrator.");
      
      abmeldeSendeDaV(_pidSBA, _atg, _asp);
      return;
    }
    
    SystemObject sysObj = _connection.getDataModel().getObject(_pidSBA);
    
    if (data != null)
    {
      AttributeGroup atg = _connection.getDataModel().getAttributeGroup (_atg);
      
      Aspect asp = _connection.getDataModel().getAspect(_asp);
      
      DataDescription dd = new DataDescription(atg, asp);

      ResultData dr = new ResultData( sysObj, dd, new Date ().getTime(), data );

      // Es wird versucht die Daten zu senden, wenn dies fehlschlgt weil das Programm
      // sich noch nicht angemeldet hat, so erfolgt die Anmeldung und danach ein 
      // erneutes Senden.
      
      // 1. Sendeversuch
      
      boolean uebertragen = true;
      
      try
      {
        _connection.sendData( dr );
        
        if (_debug)
          System.out.println("Daten gesendet 1. Versuch");
        
        uebertragen = true;
      }
      catch ( DataNotSubscribedException e1 )
      {
        // (Erneutes) Anmelden am DaV
        
        anmeldeSendeDaV( _pidSBA, _atg, _asp, _role );

        // ... und 2. Sendeversuch
        try
        {
          _connection.sendData( dr );
          
          if (_debug)
            System.out.println("Daten gesendet 2. Versuch");
          
          uebertragen = true;
        }
        catch ( DataNotSubscribedException e2 )
        {
          e2.printStackTrace();
          uebertragen = false;
        }
        catch ( SendSubscriptionNotConfirmed e2 )
        {
          e2.printStackTrace();
          uebertragen = false;
        }
      }
      catch ( SendSubscriptionNotConfirmed e1 )
      {
        e1.printStackTrace();
        uebertragen = false;
      }

      if (!uebertragen)
      {
        generiereMeldungFehler("Sende Parameter " + _atg + ": ", "Daten konnten nicht an den Steuerrechner bertragen werden\n");
      }
      
      abmeldeSendeDaV(_pidSBA, _atg, _asp);
    }
  }

  /**
   * dataRequest des ClientSenderInterface
   */
  public void dataRequest( SystemObject arg0, DataDescription arg1, byte arg2 )
  {
    if (arg2 == 1)
      _empfaengerBereit = false;
    else
      _empfaengerBereit = true;
  }

  /**
   * isRequestSupported des ClientSenderInterface
   */
  public boolean isRequestSupported( SystemObject arg0, DataDescription arg1 )
  {
    return true;
  }

  /**
   * Method zum Abmelden der Instanz am DaV
   */
  public void dispose ()
  {
    abmeldeSendeDaV( _pidSBA, _atg, _asp );
  }
  
  /**
   * Generiert eine grafische Fehlermeldung und zeigt diese an.
   * 
   * @param titel
   *          Titel der Fehlermeldung
   * @param meldung
   *          Mitteilung der Fehlermeldung
   */
  private void generiereMeldungFehler(String titel, String meldung)
  {
  	try
  	{
	    MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
	        .getActiveShell(), SWT.OK | SWT.ICON_ERROR);
	    mb.setText(titel);
	    mb.setMessage(meldung);
	    mb.open();
  	}
  	catch (Exception e) 
  	{
    }
  }
}
