/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.notwendigeschaltungen.notwendigeschaltung;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dav.DaVKommunikationSBA;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse enthlt die Parameter fr die Regelklasse "Notwendige Schaltungen", Regel "Notwendige Schaltung".<br>
 * Regel:<br>
 * Wenn ein angeforderter Bildinhalt eines WVZInhalts = "Notwendig" 
 * und "angeforderter Bildinhalt einen WVZ-Inhalts" = "Nicht darstellbar" 
 * dann alle Bildinhalte eines WVZ-Inhalts des Schaltbilds auf "entfernt" setzen 
 * und nchstes alternatives Schaltbild anfordern (gibt es keine solche, wird nichts angefordert). 
 * @version $Revision: 1.1 $ / $Date: 2013/06/25 09:27:11 $ / ($Author: ObertM $)
 * @author Swarco Traffic Systems GmbH
 * @author Matthias Obert
 *
 */
public class VerwaltungParaNotwendigeSchaltung
{
	/**
	 * Einzige Instanz der Klasse
	 */
  private static VerwaltungParaNotwendigeSchaltung _instance;

  /**
   * Debug Ausgabe
   */
  private boolean _debug = false;

  /**
   * Pid der angeforderten Attributgruppe
   */
  private String _pidAtg = "atg.sswSbaNotwendigeSchaltung";
  
  /**
   * Allgemeiner Schaltgrund fr die Regelklasse
   */
  private Aspect m_schaltgrundAllgemein;
  
  /**
   * Parametersatz fr die Anlage
   */
  private RegelNotwendigeSchaltung m_parameterAnlage = new RegelNotwendigeSchaltung ();
  
  /**
   * Parametersatz pro Anzeigequerschnitt
   */
  private HashMap<SystemObject, RegelNotwendigeSchaltung> m_parameterProAnzeigeQuerschnitt = new HashMap<SystemObject, RegelNotwendigeSchaltung>();
  
  /**
   * Parametersatz pro Anzeige
   */
  private HashMap<SystemObject, RegelNotwendigeSchaltung> m_parameterProAnzeige = new HashMap<SystemObject, RegelNotwendigeSchaltung>();
  
  /**
   * Verwaltung der Listener
   */
  private PropertyChangeSupport m_pcs = new PropertyChangeSupport (this);

  /**
   * Mutex fr den synchronisierten Zugriff auf die Datenstruktur
   */
  private Object m_mutex = new Object();

  /**
   * Pid der Anlage
   */
  private String m_anlage;
  
  /**
   * Pid des Defaultaspekts
   */
  public static String _pidDefaultAspekt = "asp.sswSbaNotwendigeSchaltung";
  
  /**
   * Pid der SBA
   */
  private String _pidSBA;

  /**
   * Privater Konstruktor der Klasse
   */
  private VerwaltungParaNotwendigeSchaltung ()
  {
  	_pidSBA = Konfig.getInstance().getPidSBA();
    
    m_schaltgrundAllgemein = Konfig.getInstance().getAspekt(_pidDefaultAspekt);

    /*
     * Daten lesen ber DaVKommunikation
     */
  	final DaVKommunikationSBA kom = DaVKommunikationSBA.getInstanz();
  	kom.addDatenArt(_pidSBA, _pidAtg, "asp.parameterSoll", ReceiverRole.receiver());
  	
  	kom.addPropertyChangeListener(new PropertyChangeListener()
		{
			@Override
			public void propertyChange(PropertyChangeEvent evt)
			{
				if (evt.getPropertyName().equals(_pidAtg))
				{
					ResultData resultData = kom.getData(_pidAtg);
					if (resultData != null)
						auswerteAttributgruppe(resultData);
				}
			}
		});

		ResultData resultData = kom.getData(_pidAtg);
		if (resultData != null)
			auswerteAttributgruppe(resultData);
  }
  
  /**
   * Methode liefert die Singleton Instanz der Klasse
   * @return Singleton Instanz der Klasse
   */
  public static VerwaltungParaNotwendigeSchaltung getInstance ()
  {
    if (_instance == null)
    {
      synchronized (VerwaltungParaNotwendigeSchaltung.class)
      {
        if (_instance == null)
        {
          _instance = new VerwaltungParaNotwendigeSchaltung ();
        }
      }
    }
    
    return _instance;
  }

  /**
   * Methode liefert den Schaltgrund fr die Regelklasse
   * @return Schaltgrund fr die Regelklasse
   */
  public Aspect getSchaltgrundAllgemein()
  {
    return m_schaltgrundAllgemein;
  }

  /**
   * Methode setzt des Schaltgrund fr dei Regelklasse
   * @param schaltgrund Schaltgrund fr dei Regelklasse
   */
  public void setSchaltgrundAllgemein( Aspect schaltgrund )
  {
    m_schaltgrundAllgemein = schaltgrund;
  }
  /**
   * Methode fgt einen Listener der Klasse hinzu
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void addPropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.addPropertyChangeListener( pcl );    
  }
  
  /**
   * Methode entfernt einen Listener aus der Klasse
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void removePropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.removePropertyChangeListener( pcl );    
  }

  /**
   * Methode benachrichtigt die Listener
   */
  private void benachrichtigeListener ()
  {
    m_pcs.firePropertyChange( "Aktualisierung", null, null );
    
    if (_debug)
      System.out.println(toString());
  }
  

  public void auswerteAttributgruppen (ResultData[] results)
  {
    int anzahl = results.length;

    // Daten aus Eingangsqueue bearbeiten      
    
    for (int i = 0; i < anzahl; i++)
    {
      ResultData dat = results[i];
  
      if (dat.getDataDescription().getAttributeGroup().getPid().equals(_pidAtg))
      {
        auswerteAttributgruppe (dat);
        
        benachrichtigeListener();
      }
    }
  }
  
  /**
   * Methode zum Auswerten der Attributgruppe 'atg.sswSbaAbsolutNotwendigeSchaltung'
   * @param dat empfangender Datensatz
   */
  private void auswerteAttributgruppe( ResultData dat )
  {
    synchronized (m_mutex)
    {
      if (!dat.hasData())
      {
        m_schaltgrundAllgemein = null;
        m_parameterAnlage.setRegelAnwenden( false);
        m_parameterProAnzeigeQuerschnitt.clear();
        m_parameterProAnzeige.clear();
        
      }
      else
      {
        Data data = dat.getData();
          
        m_schaltgrundAllgemein = (Aspect) data.getReferenceValue( "SchaltgrundAllgemein" ).getSystemObject();
        
        Data d1 = data.getItem( "AnlageRegeln" );
        auswerteAttributgruppeAnlage( d1 );
        
        Array a = data.getArray( "AnzeigeQuerschnitt" );
        auswerteAttributgruppeQuerschnitt (a);
        
        a = data.getArray( "Anzeige");
        auswerteAttributgruppeAnzeige (a);
      }
      
      m_schaltgrundAllgemein = Konfig.getInstance().getAspekt(_pidDefaultAspekt);
    }
  }

  /**
   * Methode zum Auswerten der fr die gesamte Anlage geltenden Parameter
   * @param data Daten vom DaV
   */
  private void auswerteAttributgruppeAnlage (Data data)
  {
    m_parameterAnlage.auswerteAttributgruppe ( data, getSchaltgrundAllgemein() );
  }

  /**
   * Methode zum Auswerten der Anzeigequerschnitt spezifischen Parameter
   * @param array Array mit Parametern fr die Anzeigequerschnitte
   */
  private void auswerteAttributgruppeQuerschnitt( Array array )
  {
    int anzahl = array.getLength(); 
    
    synchronized (m_parameterProAnzeigeQuerschnitt)
    {
      m_parameterProAnzeigeQuerschnitt.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        Data d1 = array.getItem( x );
        
        SystemObject anzeigeQuerschnitt = d1.getReferenceValue( "AnzeigeQuerschnitt" ).getSystemObject();
        Data d2 = d1.getItem ( "AnzeigeQuerschnittRegeln" );
        
        RegelNotwendigeSchaltung regel  = new RegelNotwendigeSchaltung ();
        regel.auswerteAttributgruppe( d2, getSchaltgrundAllgemein() );
        
        m_parameterProAnzeigeQuerschnitt.put (anzeigeQuerschnitt, regel);
      }
    }
  }

  /**
   * Methode zum Auswerten der Anzeige spezifischen Parameter
   * @param array Array mit Parametern fr die Anzeige
   */
  private void auswerteAttributgruppeAnzeige (Array array )
  {
    int anzahl = array.getLength(); 
    
    synchronized (m_parameterProAnzeige)
    {
      m_parameterProAnzeige.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        Data d1 = array.getItem( x );
        
        SystemObject anzeige = d1.getReferenceValue( "Anzeige" ).getSystemObject();
        Data d2 = d1.getItem ( "AnzeigeRegeln" );
        
        RegelNotwendigeSchaltung regel  = new RegelNotwendigeSchaltung ();
        regel.auswerteAttributgruppe( d2, getSchaltgrundAllgemein() );
        
        m_parameterProAnzeige.put (anzeige, regel );
      }
    }
  }

  /**
   * Methode fgt eine Regel fr die allgemeinen Parameter der Anlage hinzu
   * @param regelAnwenden Soll Regel angewendet werden
   * @param schaltgrund Schaltgrund unter dem diese Regel publiziert wird
   */
  public void addRegelAllgemein( boolean regelAnwenden, Aspect schaltgrund )
  {
    m_parameterAnlage.setRegelAnwenden( regelAnwenden );
  }
  
  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    s += "Parameter Notwendige Schaltung\n";
    s += "==============================\n";
    
    s += "Parameter Anlage:\n";
    s += "=================\n";
    s += m_parameterAnlage.toString();
    
    s += "Parameter pro Anzeigequerschnitt:\n";
    s += "=================================\n";
    Iterator<Entry<SystemObject, RegelNotwendigeSchaltung>> it = m_parameterProAnzeigeQuerschnitt.entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry<SystemObject, RegelNotwendigeSchaltung> entry = it.next();
      
      SystemObject anzeigeQuerschnitt = (SystemObject) entry.getKey();
      RegelNotwendigeSchaltung regel = (RegelNotwendigeSchaltung) entry.getValue();
      
      s += Tools.getObjektName( anzeigeQuerschnitt )  + "\n";
      s += regel.toString();
    }

    s += "Parameter pro Anzeige:\n";
    s += "======================\n";
    it = m_parameterProAnzeige.entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry<SystemObject, RegelNotwendigeSchaltung> entry = it.next();
      
      SystemObject anzeige = (SystemObject) entry.getKey();
      RegelNotwendigeSchaltung regel = (RegelNotwendigeSchaltung) entry.getValue();
      
      s += Tools.getObjektName( anzeige )  + "\n";
      s += regel.toString();
    }

    return s;
  }

  /**
   * Methode zum Erzeugen einer Attributgruppe mit den Parametern
   * @return Attributgruppe mit den Parametern
   */
  public Data erzeugeAttributgruppe ()
  {
	  ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection == null)
      return null;

    String pidAtg = _pidAtg;
    
    // Attributgruppe zusammensetzen und versenden
      
    AttributeGroup atg = (AttributeGroup) connection.getDataModel().getAttributeGroup(pidAtg);
    Data data = connection.createData(atg );

    data.getReferenceValue( "SchaltgrundAllgemein" ).setSystemObject( getSchaltgrundAllgemein() );
    
    Data anlageRegeln = data.getItem( "AnlageRegeln" );
    erzeugeAnlageRegeln (anlageRegeln);

    Array anzeigeQuerschnitt = data.getArray( "AnzeigeQuerschnitt" );
    erzeugeAnzeigeQuerschnittRegeln (anzeigeQuerschnitt);
    
    Array anzeige = data.getArray( "Anzeige" );
    erzeugeAnzeigeRegeln (anzeige);
    
    return data;
  }

  /**
   * Methode schreibt die Anlageregeln in ein Datenfeld einer Attributgruppe
   * @param data Feld fr die Regeln
   */
  private void erzeugeAnlageRegeln( Data data )
  {
    RegelNotwendigeSchaltung rpat = m_parameterAnlage;
    erzeugeRegeln( data, rpat );
  }

  /**
   * Methode schreibt die Anzeigequerschnittregeln in ein Array einer Attributgruppe
   * @param array Array fr die Regeln
   */
  private void erzeugeAnzeigeQuerschnittRegeln( Array array )
  {
    HashMap<SystemObject, RegelNotwendigeSchaltung> paraAq = m_parameterProAnzeigeQuerschnitt;
    int anzahlAQ = paraAq.size();
    
    array.setLength( anzahlAQ );
    
    int i=0;
    Iterator<Entry<SystemObject, RegelNotwendigeSchaltung>> it = paraAq.entrySet().iterator();
    while (it.hasNext())
    {
      Entry<SystemObject, RegelNotwendigeSchaltung> entry = it.next();
      
      SystemObject anzeigeQuerschnitt = entry.getKey();
      RegelNotwendigeSchaltung rns = entry.getValue();
      
      Data dataAQ = array.getItem( i++ );
      
      dataAQ.getReferenceValue( "AnzeigeQuerschnitt" ).setSystemObject( anzeigeQuerschnitt );
      Data arrayRegeln = dataAQ.getItem(  "AnzeigeQuerschnittRegeln" );
      
      erzeugeRegeln (arrayRegeln, rns);
    }
  }

  /**
   * Methode schreibt die Anzeigeregeln in ein Array einer Attributgruppe
   * @param array Array fr die Regeln
   */
  private void erzeugeAnzeigeRegeln( Array array )
  {
    HashMap<SystemObject, RegelNotwendigeSchaltung> paraAnzeige = m_parameterProAnzeige;
    int anzahlAnzeigen = paraAnzeige.size();
    
    array.setLength( anzahlAnzeigen );
    
    int i=0;
    Iterator<Entry<SystemObject, RegelNotwendigeSchaltung>> it = paraAnzeige.entrySet().iterator();
    while (it.hasNext())
    {
      Entry<SystemObject, RegelNotwendigeSchaltung> entry = it.next();
      
      SystemObject anzeige = entry.getKey();
      RegelNotwendigeSchaltung rns = entry.getValue();
      
      Data dataAQ = array.getItem( i++ );
      
      dataAQ.getReferenceValue( "Anzeige" ).setSystemObject( anzeige );
      Data arrayRegeln = dataAQ.getItem( "AnzeigeRegeln" );
      
      erzeugeRegeln (arrayRegeln, rns);
    }
  }

  /**
   * Methode schreibt die Regeln in ein Datenfeld
   * @param data Datenfeld in das die Regeln geschrieben werden solle
   * @param rns Regeln
   */
  private void erzeugeRegeln( Data data, RegelNotwendigeSchaltung rns )
  {
    data.getReferenceValue( "SchaltgrundRegel" ).setSystemObject( rns.getSchaltgrundRegel() );
    data.getTextValue ( "RegelAnwenden" ).setText( rns.isRegelAnwenden() ? "Ja" : "Nein" );
  }

  /**
   * Methode bernimmt die Parameter fr eine bestimmte Pid
   * @param pid Pid
   * @param regeln Regeln
   */
	public void setParameter(String pid, RegelNotwendigeSchaltung regeln)
  {
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    /*
     * Parameter Anlage
     */
    if (pid.equals(_pidSBA))
    {
      m_parameterAnlage = regeln;
    }
    else if (connection != null)
    {
    	/*
    	 * Prfen ob Anzeigequerschnitt oder Anzeige
    	 */
    	
    	SystemObject so = connection.getDataModel().getObject(pid);
    	if (so != null)
    	{
    		if (so.getType().getPid().equals("typ.anzeigeQuerschnitt"))
    		{
    			if (regeln == null)
    			  m_parameterProAnzeigeQuerschnitt.remove(so);
    			else
    			  m_parameterProAnzeigeQuerschnitt.put(so, regeln);
    		}

    		if (so.getType().getPid().equals("typ.anzeige"))
    		{
    			if (regeln == null)
    			  m_parameterProAnzeige.remove (so);
    			else
    			  m_parameterProAnzeige.put(so, regeln);
    		}
    	}
    }
    
    benachrichtigeListener();
  }
	
  /**
   * Methode liefert die Parameter als Attributgruppe fr den DaV
   * @return Attributgruppe
   */
  public Data getData ()
  {
  	return erzeugeAttributgruppe();
  }

  /**
   * Methode liefert die Parameter zu einer bestimmten Pid
   * @param pid Pid des gesuchten Objekts, dies kann die Pid der SBA, die Pid eines Anzeigequerschnitts oder
   * die Pid einer Anzeige sein.
   * @return Parameter zu der Pid
   */
	public RegelNotwendigeSchaltung getParameter(String pid)
  {
    /*
     * Parameter Anlage
     */
    if (pid.equals(_pidSBA))
      return m_parameterAnlage;
    
    /*
     * Parameter Anzeigequerschnitt
     */
    for (SystemObject so : m_parameterProAnzeigeQuerschnitt.keySet())
    {
      if (pid.equals(so.getPid()))
        return m_parameterProAnzeigeQuerschnitt.get(so);
    }
    
    /*
     * Parameter Anzeige
     */
    for (SystemObject so : m_parameterProAnzeige.keySet())
    {
      if (pid.equals(so.getPid()))
        return m_parameterProAnzeige.get(so);
    }
    
    /*
     * Parametersatz noch nicht definiert
     */
   
    return null;
  }

	public HashMap<SystemObject, RegelNotwendigeSchaltung> getParameterProAnzeigeQuerschnitt()
  {
		return m_parameterProAnzeigeQuerschnitt;
  }

	public HashMap<SystemObject, RegelNotwendigeSchaltung> getParameterProAnzeige()
  {
	  return m_parameterProAnzeige;
  }
  

}
