/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.parametereditor.editor;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.NumberFormat;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.datenidentifikation.DatenidentifikationGtm;
import de.bsvrz.buv.rw.basislib.datenidentifikation.GtmListeComposite;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.IntegerAttributeType;
import de.bsvrz.dav.daf.main.config.IntegerValueRange;
import de.bsvrz.dav.daf.main.config.IntegerValueState;
import de.bsvrz.dav.daf.main.config.ReferenceAttributeType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.pat.sysbed.dataEditor.DataEditorPanel;

/**
 * {@link Data}-Editor in SWT. Entspricht von der Funktionalitt her dem
 * {@link DataEditorPanel}
 * 
 * @author HofmannJ
 * 
 */
public class DataEditorComposite extends Composite
{
	/** ein Zahlenformat */
	private static final NumberFormat _integerNumberFormat = NumberFormat
			.getNumberInstance();

	/** Genauigkeit des Zahlenformats */
	private static final NumberFormat _precisionTestNumberFormat;

	/**
	 * Hintergrundfarbe eines Attributwertes, der nicht definiert ist und nicht
	 * ber den Datenvertiler verschickt werden kann.
	 */
	private static final Color _backgroundUndefinedValue = new Color(PlatformUI
			.getWorkbench().getDisplay(), 255, 0, 0);

	/**
	 * Hintergrundfarbe eines Attributwertes, der ber den Datenvertiler
	 * verschickt werden kann.
	 */
	private static final Color _backgroundColorDefinedValue = new Color(
			PlatformUI.getWorkbench().getDisplay(), 255, 255, 255);

	/**
	 * String der ausgegeben wird, wenn der Attributwert "undefiniert" ist. Der
	 * String entspricht dabei nicht dem wahren undefiniert Wert.
	 */
	private static final String _undefinedString = "_Undefiniert_";

	/**
	 * String, der in Comboboxen angezeigt wird und anzeigt, dass der Default-Wert
	 * benutzt werden soll.
	 */
	private static final String _defaultValueString = "Default-Wert";

	/**
	 * Das Zahlenformat erhlt eine Formatierung.
	 */
	static
	{
		_integerNumberFormat.setMinimumIntegerDigits(1);
		_integerNumberFormat.setMaximumIntegerDigits(999);
		_integerNumberFormat.setGroupingUsed(false);
		DecimalFormatSymbols symbols = new DecimalFormatSymbols();
		symbols.setDecimalSeparator(',');
		_precisionTestNumberFormat = new DecimalFormat("0.#", symbols);
		_precisionTestNumberFormat.setMaximumFractionDigits(999);
	}

	/** die Verbindung zum Datenverteiler */
	private ClientDavInterface _connection = null;

	/** speichert die aktuellen Daten */
	private Data _data;

	/** gibt an, ob die dargestellten Felder editierbar sein sollen */
	private boolean _editable = true;

	/**
	 * Lokaler Verweis
	 */
	private DataEditorComposite _this = null;

	/**
	 * Speichert zu einem {@link ComboViewer} einen zugehrigen
	 * {@link ISelectionChangedListener}. Dieser Listener muss vom
	 * {@link ComboViewer} detached werden, wenn die Aktion hinter dem Listener
	 * ausgefhrt wird, da diese u.U. eigene Selektionen auf dem
	 * {@link ComboViewer} durchfhrt. Ist diese Aktion am Ende, attached sie den
	 * Listener wieder.
	 */
	private HashMap<ComboViewer, ISelectionChangedListener> _mapComboListener = new HashMap<ComboViewer, ISelectionChangedListener>();

	/**
	 * In diesem {@link ScrolledComposite} findet die Darstellung des
	 * {@link DataEditorComposite} statt und dieses {@link ScrolledComposite} muss
	 * ber nderungen der Gre des {@link DataEditorComposite} informiert
	 * werden.
	 */
	private ScrolledComposite _scrolledComposite = null;

	/**
	 * Konstruktor.
	 * 
	 * @param parent
	 *          Eltern-{@link Composite}, auf dessen Basis die Darstellung
	 *          erfolgen soll. Das Eltern-{@link Composite} sollte ein
	 *          {@link ScrolledComposite} sein.
	 * @param style
	 *          SWT-Stil
	 * @param connection
	 *          Verbindung zum DaV.
	 */
	public DataEditorComposite(Composite parent, int style,
			ClientDavInterface connection)
	{
		super(parent, style);

		if (parent instanceof ScrolledComposite)
		{
			this._scrolledComposite = (ScrolledComposite) parent;
		}

		this._connection = connection;
		this._this = this;

		initialize();
	}

	/**
	 * Fhrt die Basis-Initialisierung des Composites durch
	 */
	private void initialize()
	{
		setLayout(new GridLayout());
	}

	public void setDataDav(Data data)
	{
		this._data = data;

		Control[] children = this.getChildren();
		for (Control control : children)
		{
			control.dispose();
		}

		if (data == null)
		{
			Text text = new Text(this, SWT.BORDER);
			text.setBackground(_backgroundUndefinedValue);
			text.setText("keine Daten");
		}
		else
		{
			createBox(this, _data);
		}
	}

	/**
	 * Gibt die Daten zurck, die aktuell angezeigt werden.
	 * 
	 * @return die aktuellen Daten
	 */
	public Data getDataDav()
	{
		return _data;
	}

	/**
	 * Hierber kann bestimmt werden, ob die angezeigten Textfelder, etc.
	 * editierbar sind, oder nicht.
	 * 
	 * @param editable
	 *          gibt an, ob die angezeigten Komponenten editierbar sein sollen
	 */
	protected void setEditable(boolean editable)
	{
		_editable = editable;
	}

	protected boolean isEditable()
	{
		return _editable;
	}

	/**
	 * Erzeugt eine neue Gruppe zur Darstellung eines {@link Data}. Ruft sich
	 * selbstttig rekursiv auf.
	 * 
	 * @param parent
	 *          Eltern-{@link Composite}, auf dessen Basis die Darstellung
	 *          erfolgen soll.
	 * @param data
	 *          {@link Data} zur Darstellung in der neuen Gruppe.
	 */
	private void createBox(Composite parent, final Data data)
	{
		if (data.isPlain())
		{
			Composite composite = new Composite(parent, SWT.NONE);
			GridData gdComposite = new GridData();
			gdComposite.grabExcessHorizontalSpace = true;
			gdComposite.horizontalAlignment = GridData.FILL;
			composite.setLayoutData(gdComposite);

			/*
			 * Name der Box setzen
			 */
			Label labelNameBox = new Label(composite, SWT.NONE);
			labelNameBox.setText(data.getName() + ": ");

			if (data.getAttributeType() instanceof ReferenceAttributeType)
			{
				ReferenceAttributeType att = (ReferenceAttributeType) data
						.getAttributeType();
				SystemObjectType objectType = att.getReferencedObjectType();
				final List<SystemObjectType> types = new LinkedList<SystemObjectType>();
				if (objectType == null)
				{
					DataModel configuration = _connection.getDataModel();
					types.add(configuration.getType("typ.konfigurationsObjekt"));
					types.add(configuration.getType("typ.dynamischesObjekt"));
					// configuration.getObjects(configurationAreas, systemObjectTypes,
					// objectTimeSpecification)
				}
				else
				{
					types.add(objectType);
				}

				/*
				 * Infotext zur Referenz
				 */
				final Text textBox = new Text(composite, SWT.BORDER);
				textBox.setEditable(false);
				final Label suffixBox = new Label(composite, SWT.NONE);

				if (data.isDefined())
				{
					textBox.setText(data.asTextValue().getValueText());
					suffixBox.setText(data.asTextValue().getSuffixText());
				}
				else
				{
					textBox.setText(_undefinedString);
					textBox.setBackground(_backgroundUndefinedValue);
					suffixBox.setText(_undefinedString);
				}

				Button buttonRefAend = new Button(composite, SWT.PUSH);
				buttonRefAend.setText("Referenz ndern");
				buttonRefAend.setEnabled(false);

				if (_editable)
				{
					buttonRefAend.setEnabled(true);
					buttonRefAend.addSelectionListener(new SelectionAdapter()
					{
						public void widgetSelected(org.eclipse.swt.events.SelectionEvent e)
						{

							// PreselectionDialog dialog = new PreselectionDialog(
							// "Objektauswahl", null, null, types);
							// dialog.setMaximumSelectedObjects(1);
							// dialog.setMaximumSelectedAspects(0);
							// dialog.setMaximumSelectedAttributeGroups(0);
							// try
							// {
							// dialog.setSelectedObject(data.asReferenceValue()
							// .getSystemObject());
							// }
							// catch (Exception ignore)
							// {
							// }
							// if (dialog.show())
							// {
							// data.asReferenceValue().setSystemObject(
							// (SystemObject) dialog.getSelectedObjects().get(0));
							// if (data.isDefined())
							// {
							// textBox.setText(data.asTextValue().getValueText());
							// suffixBox.setText(data.asTextValue().getSuffixText());
							// textBox.setBackground(_backgroundColorDefinedValue);
							// }
							// else
							// {
							// textBox.setText(_undefinedString);
							// textBox.setBackground(_backgroundUndefinedValue);
							// }
							//
							// }

							// DatenidentifikationNeu ident = new
							// DatenidentifikationNeu(PlatformUI.getWorkbench().getDisplay().getActiveShell(),
							// _connection);
							// ident.oeffne(Darstellung.OBJEKTELISTE_OBJEKTTYP);

							DatenidentifikationGtm dia = new DatenidentifikationGtm(
									PlatformUI.getWorkbench().getDisplay().getActiveShell(), null);
							LinkedList<SystemObject> so = dia.oeffnen(types,
									GtmListeComposite._darstlungSystemObjekteObjektTypen);

							if (so.size() > 0)
							{
								data.asReferenceValue().setSystemObject(so.getFirst());
								if (data.isDefined())
								{
									textBox.setText(data.asTextValue().getValueText());
									suffixBox.setText(data.asTextValue().getSuffixText());
									textBox.setBackground(_backgroundColorDefinedValue);
								}
								else
								{
									textBox.setText(_undefinedString);
									textBox.setBackground(_backgroundUndefinedValue);
								}
							}

						}
					});
				}

				/*
				 * Layout
				 */
				GridLayout gridLayout = new GridLayout(4, false);
				composite.setLayout(gridLayout);

				// TextBox
				GridData gdTextBox = new GridData();
				gdTextBox.grabExcessHorizontalSpace = true;
				gdTextBox.horizontalAlignment = GridData.FILL;
				textBox.setLayoutData(gdTextBox);
			}
			else if (data.getAttributeType() instanceof IntegerAttributeType)
			{
				if (_editable)
				{
					// true == ComboBox
					final Combo comboBox = new Combo(composite, SWT.NONE);
					final ComboViewer cvComboBox = new ComboViewer(comboBox);

					final Label suffixBox = new Label(composite, SWT.NONE);

					IntegerAttributeType att = (IntegerAttributeType) data
							.getAttributeType();

					List<IntegerValueState> states = att.getStates();
					for (Iterator<IntegerValueState> statesIterator = states.iterator(); statesIterator
							.hasNext();)
					{
						IntegerValueState state = (IntegerValueState) statesIterator.next();
						cvComboBox.add(state.getName());
					}

					IntegerValueRange range = att.getRange();
					if (range != null)
					{
						comboBox.setEnabled(true);

						final long unscaledMinimum = range.getMinimum();
						final long unscaledMaximum = range.getMaximum();
						final double conversionFactor = range.getConversionFactor();

						cvComboBox
								.add(getScaledValueText(unscaledMinimum, conversionFactor));
						cvComboBox
								.add(getScaledValueText(unscaledMaximum, conversionFactor));
					}

					// Wert, der gesetzt werden muss
					if (data.isDefined())
					{
						comboBox.setText(data.asTextValue().getValueText());

						// Der Wert des Attributes ist definiert
						StructuredSelection ss = new StructuredSelection(data.asTextValue()
								.getValueText());
						cvComboBox.setSelection(ss, true);

					}
					else
					{
						// Damit wenn kein Range vorhanden ist, eine Eingabe mglich ist
						comboBox.setEnabled(true);
						comboBox.setBackground(_backgroundUndefinedValue);
						cvComboBox.setSelection(new StructuredSelection(_undefinedString));

					}

					cvComboBox.add(_defaultValueString);

					ISelectionChangedListener cvListener = new ISelectionChangedListener()
					{
						@Override
						public void selectionChanged(SelectionChangedEvent event)
						{
							comboBoxAttributeModifiziertWertPruefung(cvComboBox, suffixBox,
									data);
						}
					};

					cvComboBox.addSelectionChangedListener(cvListener);

					comboBox.addModifyListener(new ModifyListener()
					{
						@Override
						public void modifyText(ModifyEvent e)
						{

							if (e.getSource() instanceof Combo)
							{
								Combo sourceCombo = (Combo) e.getSource();
								String text = sourceCombo.getText();
								data.asTextValue().setText(text);

								if (text.length() > 0)
								{
									cvComboBox.getCombo().setBackground(
											_backgroundColorDefinedValue);
								}
								else
								{
									cvComboBox.getCombo()
											.setBackground(_backgroundUndefinedValue);
								}
							}
						}
					});

					_mapComboListener.put(cvComboBox, cvListener);

					/*
					 * Editable?
					 */
					if (!_editable)
					{
						comboBox.setEnabled(false);
					}

					/*
					 * Layout
					 */
					GridLayout gridLayout = new GridLayout(3, false);
					composite.setLayout(gridLayout);

					// TextBox
					GridData gdComboBox = new GridData();
					gdComboBox.grabExcessHorizontalSpace = true;
					gdComboBox.horizontalAlignment = GridData.FILL;
					comboBox.setLayoutData(gdComboBox);
				}
				else
				{
					// ist false -> nicht editierbares Textfeld
					Text textField = new Text(composite, SWT.BORDER);
					textField.setEditable(false);
					textField.setText(data.asTextValue().getValueText());

					/*
					 * editable
					 */
					if (!_editable)
					{
						textField.setEditable(false);
					}

					/*
					 * Layout
					 */
					GridLayout gridLayout = new GridLayout(2, false);
					composite.setLayout(gridLayout);

					// TextField
					GridData gdTextFeld = new GridData();
					gdTextFeld.grabExcessHorizontalSpace = true;
					gdTextFeld.horizontalAlignment = GridData.FILL;
					textField.setLayoutData(gdTextFeld);
				}
			}
			else
			{
				final Text textBox;
				final Label suffixBox;

				if (data.isDefined())
				{
					textBox = new Text(composite, SWT.BORDER);
					textBox.setText(data.asTextValue().getValueText());
					suffixBox = new Label(composite, SWT.NONE);
					suffixBox.setText(data.asTextValue().getSuffixText());
				}
				else
				{
					textBox = new Text(composite, SWT.BORDER);
					textBox.setText(_undefinedString);
					textBox.setBackground(_backgroundUndefinedValue);
					suffixBox = new Label(composite, SWT.BORDER);
				}

				/*
				 * editable
				 */
				textBox.setEditable(_editable); // gibt an, ob dieses Feld die Daten nur
				// anzeigt

				textBox.addFocusListener(new FocusListener()
				{
					@Override
					public void focusGained(FocusEvent e)
					{
					}

					@Override
					public void focusLost(FocusEvent e)
					{
						textBoxAttributeModifiedPruefung(textBox, suffixBox, data);
					}
				});

				textBox.addListener(SWT.DefaultSelection, new Listener()
				{
					public void handleEvent(Event e)
					{
						textBoxAttributeModifiedPruefung(textBox, suffixBox, data);
					}
				});

				/*
				 * Layout
				 */
				GridLayout gridLayout = new GridLayout(3, false);
				composite.setLayout(gridLayout);

				// TextField
				GridData gdTextFeld = new GridData();
				gdTextFeld.grabExcessHorizontalSpace = true;
				gdTextFeld.horizontalAlignment = GridData.FILL;
				textBox.setLayoutData(gdTextFeld);
			}
		}
		else
		{
			/*
			 * Urlasser ausfiltern
			 */
			if (!data.getName().equals("Urlasser"))
			{
				final Group arrayBox = new Group(parent, SWT.NONE);
				arrayBox.setText(data.getName() + ": ");

				final GridData gdBox = new GridData();
				gdBox.grabExcessHorizontalSpace = true;
				gdBox.horizontalAlignment = GridData.FILL;
				arrayBox.setLayoutData(gdBox);
				arrayBox.setLayout(new GridLayout());

				if (data.isArray())
				{
					final Composite arrayHeaderBox = new Composite(arrayBox, SWT.NONE);
					GridLayout glArrayHeaderBox = new GridLayout(2, false);
					arrayHeaderBox.setLayout(glArrayHeaderBox);

					GridData gdArrayHeaderBox = new GridData();
					gdArrayHeaderBox.grabExcessHorizontalSpace = true;
					gdArrayHeaderBox.horizontalAlignment = GridData.FILL;
					arrayHeaderBox.setLayoutData(gdArrayHeaderBox);

					Label labelSpinner = new Label(arrayHeaderBox, SWT.NONE);
					labelSpinner.setText("Arraygre: ");

					final Spinner spinner;
					final Label labelArraySize;

					Data.Array array = data.asArray();
					if (array.isCountVariable() && _editable)
					{
						spinner = new Spinner(arrayHeaderBox, SWT.READ_ONLY);
						spinner.setMinimum(0);
						spinner.setMaximum(Integer.MAX_VALUE);
						spinner.setIncrement(1);
						spinner.setPageIncrement(1);
						spinner.setSelection(array.getLength());

						if (array.isCountLimited())
						{
							spinner.setMaximum(new Integer(array.getMaxCount()));
							spinner.setToolTipText("Minimum: 0, Maximum: "
									+ array.getMaxCount());
						}
						else
						{
							spinner.setToolTipText("Minimum: 0, Maximum: "
									+ Integer.MAX_VALUE);
						}

						spinner.addModifyListener(new ModifyListener()
						{
							@Override
							public void modifyText(ModifyEvent e)
							{
								// arrayBox.setVisible(false);

								data.asArray().setLength(spinner.getSelection());

								leereCompositeMitAusnahme(arrayBox, arrayHeaderBox);

								@SuppressWarnings("unchecked")
								Iterator iterator = data.iterator();
								while (iterator.hasNext())
								{
									Data subData = (Data) iterator.next();

									createBox(arrayBox, subData);
									arrayBox.layout();

									_this.layout(true);
								}

								// arrayBox.setVisible(true);
							}
						});

						/*
						 * Editable
						 */
						if (spinner != null)
						{
							spinner.setEnabled(_editable);
						}
					}
					else
					{
						if (!array.isCountVariable())
						{
							array.setLength(array.getMaxCount());
							labelArraySize = new Label(arrayHeaderBox, SWT.NONE);
							labelArraySize.setText("" + array.getLength());
						}
					}

				}

				@SuppressWarnings("unchecked")
				Iterator iterator = data.iterator();
				while (iterator.hasNext())
				{
					Data subData = (Data) iterator.next();
					createBox(arrayBox, subData);
				}
			}
		}

		if (_scrolledComposite != null)
		{
			_scrolledComposite.setMinSize(this.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		}
		this.layout();
	}

	/**
	 * Entfernt von einem {@link Composite} alle Kinder. Es knnen Ausnahmen
	 * definiert werden.
	 * 
	 * @param toClearComposite
	 *          Zu leerendes {@link Composite}.
	 * @param exceptionComposite
	 *          Ausnahme-{@link Composite} oder <code>null</code>, wenn alle
	 *          entfernt werden sollen.
	 */
	private void leereCompositeMitAusnahme(Composite toClearComposite,
			Composite exceptionComposite)
	{
		Control[] children = toClearComposite.getChildren();
		for (Control control : children)
		{
			if (exceptionComposite != null)
			{
				if (control != exceptionComposite)
				{
					control.dispose();
				}
			}
			else
			{
				control.dispose();
			}
		}
	}

	private String getScaledValueText(final long unscaledValue,
			double conversionFactor)
	{

		if (conversionFactor == 1)
		{
			return String.valueOf(unscaledValue);
		}
		else
		{
			int precision = 0;
			synchronized (_integerNumberFormat)
			{
				String formatted = _precisionTestNumberFormat.format(conversionFactor);
				int kommaPosition = formatted.lastIndexOf(',');
				if (kommaPosition >= 0)
					precision = formatted.length() - kommaPosition - 1;
				_integerNumberFormat.setMinimumFractionDigits(precision);
				_integerNumberFormat.setMaximumFractionDigits(precision);
				return _integerNumberFormat.format(unscaledValue * conversionFactor);
			}
		}
	}

	/**
	 * berprft, ob der eingegebene Wert auch ein zulssiger Wert ist. Bevor die
	 * Aktionen durchgefhrt werden, wird vom jeweiligen {@link ComboViewer} der
	 * registrierte {@link ISelectionChangedListener} detached. Am Ende der
	 * Methode wird der Listener wieder attached. Da dies Methode selbst
	 * Selektionen im ComboViewer ndern kann, wrde es sonst zu Endlos-Loops im
	 * SWT und damit verbunden zu Stack-Fehlern und Speicherlecks fhren.
	 * 
	 * @param cvComboBox
	 *          Komponente, wo der neue Wert eingegeben wurde
	 * @param suffixBox
	 *          das Feld mit dem Suffix
	 * @param data
	 *          die modifizierten Daten
	 */
	private void comboBoxAttributeModifiziertWertPruefung(ComboViewer cvComboBox,
			Label suffixBox, Data data)
	{
		cvComboBox
				.removeSelectionChangedListener(_mapComboListener.get(cvComboBox));
		String text = null;

		ISelection selection = cvComboBox.getSelection();
		if (selection instanceof StructuredSelection)
		{
			StructuredSelection ss = (StructuredSelection) selection;

			if (ss.getFirstElement() != null)
			{
				text = ss.getFirstElement().toString();
			}
			else
			{
				text = cvComboBox.getCombo().getText();
			}

			// System.out.println("TEXT: " + text);
		}

		try
		{
			if (text.equals(_defaultValueString) || text.equals(_undefinedString))
			{
				data.setToDefault();
			}
			else
			{
				data.asTextValue().setText(text);
			}

			if (!data.isDefined())
			{
				cvComboBox.getCombo().setBackground(_backgroundUndefinedValue);
				StructuredSelection ss = new StructuredSelection(_undefinedString);
				cvComboBox.setSelection(ss, true);
			}
			else
			{
				StructuredSelection ss = new StructuredSelection(data.asTextValue()
						.getValueText());
				cvComboBox.setSelection(ss, true);

				suffixBox.setText(data.asTextValue().getSuffixText());

				cvComboBox.getCombo().setBackground(_backgroundColorDefinedValue);
			}
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}

		cvComboBox.addSelectionChangedListener(_mapComboListener.get(cvComboBox));
	}

	/**
	 * berprft, ob der eingegebene Wert auch ein zulssiger Wert ist.
	 * 
	 * @param textBox
	 *          Komponente, wo der neue Wert eingegeben wurde
	 * @param suffixBox
	 *          das Feld mit dem Suffix
	 * @param data
	 *          die modifizierten Daten
	 */
	private void textBoxAttributeModifiedPruefung(final Text textBox,
			final Label suffixBox, final Data data)
	{
		String text = textBox.getText();
		try
		{
			if (text.equals(_defaultValueString) || text.equals(_undefinedString))
			{
				data.setToDefault();
			}
			else
			{
				data.asTextValue().setText(text);
			}

			if (!data.isDefined())
			{
				textBox.setBackground(_backgroundUndefinedValue);
				textBox.setText(_undefinedString);
			}
			else
			{
				textBox.setText(data.asTextValue().getValueText());
				suffixBox.setText(data.asTextValue().getSuffixText());
				textBox.setBackground(_backgroundColorDefinedValue);
			}
		}
		catch (Exception ex)
		{
			// Wenn beim setzen des Textes ein Fehler Auftritt, dann wird der Wert auf
			// "undefiniert" gesetzt
			// und der Benutzer muss sich Gedanken um den Wert machen
			textBox.setBackground(_backgroundUndefinedValue);
			textBox.setText(_undefinedString);
			ex.printStackTrace();
		}
	}

}
