/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.sonstiges.shell;

import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

/**
 * Liefert Funktionen zum Ausrichten von {@link Shell}s auf dem {@link Display}
 * bzw. zum Ausrichten von Child {@link Rectangle}-Bounds auf Parent
 * {@link Rectangle}-Bounds.
 * 
 * @author HofmannJ
 * 
 */
public class ShellPosition
{

	/**
	 * Berechnet die zentrierte Position eines Child-Elementes auf einem
	 * Parent-Element. Es wird hier zu mit den "Bounds" der jeweiligen Element in
	 * Form eines {@link Rectangle}s gearbeitet. Es wird verhindert, dass
	 * Bereiche des zentrierten Bounds auerhalb der Flche des {@link Display}s
	 * landen.
	 * 
	 * @param parent
	 *          {@link Rectangle} der Bounds des Eltern-Elementes, auf dem das
	 *          Kind-Element zentriert positioniert werden soll.
	 * @param child
	 *          {@link Rectangle} der Bounds des Kind-Elementes, welches auf dem
	 *          Eltern-Element zentriert werden soll.
	 * @return Neues {@link Rectangle} der Bounds des Kindelementes, zentriert auf
	 *         dem Eltern-Element.
	 */
	public static Rectangle getBoundsChildCenteredOnParent(Rectangle parent,
			Rectangle child)
	{
		Rectangle re = null;
		int x = ((parent.width - child.width) / 2) + parent.x;
		int y = ((parent.height - child.height) / 2) + parent.y;

		re = new Rectangle(x, y, child.width, child.height);

		Rectangle boundsDisplay = PlatformUI.getWorkbench().getDisplay()
				.getBounds();

		/*
		 * Grer als die maximale Breite des Displays?
		 */
		if ((re.x + re.width) > boundsDisplay.width)
		{
			re.x = boundsDisplay.width - re.width;
		}

		/*
		 * Kleiner 0?
		 */
		if (re.x < 0)
		{
			re.x = 0;
		}

		/*
		 * Grer als die maximale Hhe des Displays?
		 */
		if ((re.y + re.height) > boundsDisplay.height)
		{
			re.y = boundsDisplay.height - re.height;
		}

		/*
		 * Kleiner 0?
		 */
		if (re.y < 0)
		{
			re.y = 0;
		}

		return re;
	}

	/**
	 * Positioniert eine {@link Shell} zentriert auf dem Bildschirm.
	 * 
	 * @param shell
	 *          Auf dem Bildschirm zu zentrierende {@link Shell}.
	 */
	public static void positionShellCenteredOnActualDisplay(Shell shell)
	{
		shell.setBounds(getBoundsChildCenteredOnParent(PlatformUI.getWorkbench()
				.getDisplay().getBounds(), shell.getBounds()));
	}

	/**
	 * Positoniert eine {@link Shell} mit der linken, oberen Ecke beim Mauscursor.
	 * Es kann kein Bereich der {@link Shell} ausserhalb des aktuellen
	 * {@link Display}-Bereiches liegen.
	 * 
	 * @param shell
	 *          Beim Mauscursor zu positionierende {@link Shell}.
	 */
	public static void positionShellAtMouseCursor(Shell shell)
	{
		Display display = shell.getDisplay();
		if (display != null)
		{
			int displayWidth = display.getBounds().width;
			int displayHeight = display.getBounds().height;

			Rectangle bounds = shell.getBounds();
			Point cursorLocation = display.getCursorLocation();

			int y = cursorLocation.y;
			if ((cursorLocation.y + bounds.height) > displayHeight)
			{
				y = displayHeight - bounds.height;
				if (y < 0)
				{
					y = 0;
				}
			}

			int x = cursorLocation.x;
			if ((cursorLocation.x + bounds.width > displayWidth))
			{
				x = displayWidth - bounds.width;
				if (x < 0)
				{
					x = 0;
				}
			}

			Rectangle newBounds = new Rectangle(x, y, bounds.width, bounds.height);
			shell.setBounds(newBounds);
		}

	}

}
